package Gm_Core;

###############################################################################
# Greymatter 1.7.1                            
# Copyright (c)2000-2007, The Greymatter team 
# http://greymatterforum.proboards82.com/     
# By possessing this software, you agree not to hold the author responsible for 
# any problems that may arise from your installation or usage of Greymatter 
# itself, or from any content generated by yourself or others through the use of 
# this program.  You <I>may</I> freely modify and redistribute this program, so 
# long as every copyright notice (including in this manual and in the Greymatter 
# code) remains fully intact.  Finally, you may <I>not</I> sell or in any way 
# make a financial profit from this program, either in original or modified form.
# Your possession of this software signifies that you agree to these terms; 
# please delete your copy of this software if you don't agree to these terms.
# Original Creators Noah Grey
###############################################################################

#==============================================================================
# Note no exporting because items from this package should be fully qualified
## Gm Core
# This package groups functions that deal with the core
# functionality of GreyMatter:
# - Amin screen functionality
# - 
#
# CONVENTIONS
# 1. Return a string rather than printing, this is just more elegant.  Leave prints
#   to the calling subroutine, if possible
# 2. Private subroutines should start with the '_' character.  By private I mean
#   it will never be called outside of this package.
# 3. If you have more than 1 or 2 parameters, especially if they are not required,
#   use named parameters such as in createRadioButton.  By putting stuff in a 
#   hash we gain the flexibility to add more optional parameters without having
#   to pass in '' placeholders or modify existing code.
# 4. Subs starting with 'create' should return a string.
# 5. Subs starting with 'display' are expected to contain print statements
# 6. Use ' and " where appropriate.  If you don't have any variables or newlines
# then use ', its quicker and cleaner.  
# 7. Don't put all text on one huge long line.  It messes with some programs
# that don't do line wraps.  

use strict;
use warnings;

use Gm_Utils qw( println );
use Gm_Constants;
use Gm_Storage;
use Gm_Web;

# Global vars that are highly used, maybe make local sometime
my $CONFIGS = Gm_Storage::getConfigs( errHandler=>\&Gm_Web::displayAdminErrorExit );
my $TEMPLATES = Gm_Storage::getTemplates( errHandler=>\&Gm_Web::displayAdminErrorExit );

# ---------------
# write to cp log
# ---------------
## Write to Control Panel Log
# shorthand to write to the control panel log, used by admin
# or user, but must pass handler if want error to look right
# ARGS1 = message to write to control panel
# (opt) ARGS2 = error handler
# DEPRECATES: gm_writetocplog
sub writeToCplog {
	my $writetologtext = shift(@_);
	my $errHandler = shift(@_) || \&Gm_Web::displayAdminErrorExit;

	if ($CONFIGS->{'gmkeeplog'} eq Gm_Constants::YES) {
		my ($gmdate) = Gm_Utils::getStdDate( $CONFIGS->{'gmserveroffset'} );
		my $authorIp = $ENV{'REMOTE_ADDR'};
		## A little about this regex:
		# its looking for <?, <%, <script, and <iframe tags, notice the funky 
		# \s*? regex, it basically will look for any or no whitespace after the < char
		if( Gm_Utils::hackTest( $writetologtext ) ){
			$writetologtext = '<b><font COLOR="#dd0000">[Aborted log - '.
			'Hack Attempt]</font></b> by <b>'.$ENV{'REMOTE_ADDR'}.'</b>';
		}
	
		Gm_Storage::addLogMessage( msg=>"<font size=\"1\">[$gmdate] [$authorIp]".
			"</font> $writetologtext", errHandler=>$errHandler );
	}
} 





# ---------------------------------
# translate entry templates
# ---------------------------------
## takes all the templates setup by the getEntryVariables and applies them to
# and gm template tokens in the entry
# ARG entryVars => the output of getEntryVariables
# (opt) ARG errHandler => the error handler used to display errors
# RETURN: a hash reference that contains the variables
# DEPRECATES: gm_formatentry
# sub translateEntryTemplates {
# 	my (%params) = @_;
# 
# 	## Setting up vars
# 	my $entrygetreturn = $params{'template'};
# 	my $entryVars = $params{'entryVars'};
# 	my $errorHandler = $params{'errHandler'} || \&Gm_Utils::gmWarn;
#  
# # $domain = $cgiwebpath;
# 	my $domain = $CONFIGS->{'gmcgiwebpath'}
# 	$domain =~ s@http://([^\/]+)?.*@$1@;
# 
# 	my $entryreturn = $entrygetreturn;
# 
# # $entryreturn =~ s/\|\*\|/\n/g;  ## WHY IS THIS COMMENTED OUT?  I DIDN'T DO THIS
# 
# ## TODO: MAKE GENERIC TRANSLATE ENTRY, THIS IS NOT ENTRY SPECIFIC
# 	if( ($entryreturn =~ m/{{header}}/i) || ($entryreturn =~ m/{{footer}}/i) || 
# 		($entryreturn =~ m/{{sidebar}}/i) ){
# # 		$entryreturn =~ s/{{header}}/$gmheadertemplate/gi;
# # 		$entryreturn =~ s/{{footer}}/$gmfootertemplate/gi;
# # 		$entryreturn =~ s/{{sidebar}}/$gmsidebartemplate/gi;
# 		$entryreturn =~ s/{{header}}/$TEMPLATES->{'gmheadertemplate'}/gi;
# 		$entryreturn =~ s/{{footer}}/$TEMPLATES->{'gmfootertemplate'}/gi;
# 		$entryreturn =~ s/{{sidebar}}/$TEMPLATES->{'gmsidebartemplate'}/gi;
# 	}
# 
# ## TODO: MAKE GENERIC TRANSLATE ENTRY, THIS IS NOT ENTRY SPECIFIC
# 	if ($entryreturn =~ m/{{custom/i) {
# # 		$entryreturn =~ s/{{customone}}/$gmcustomonetemplate/gi;
# # 		$entryreturn =~ s/{{customtwo}}/$gmcustomtwotemplate/gi;
# # 		$entryreturn =~ s/{{customthree}}/$gmcustomthreetemplate/gi;
# # 		$entryreturn =~ s/{{customfour}}/$gmcustomfourtemplate/gi;
# # 		$entryreturn =~ s/{{customfive}}/$gmcustomfivetemplate/gi;
# # 		$entryreturn =~ s/{{customsix}}/$gmcustomsixtemplate/gi;
# # 		$entryreturn =~ s/{{customseven}}/$gmcustomseventemplate/gi;
# # 		$entryreturn =~ s/{{customeight}}/$gmcustomeighttemplate/gi;
# # 		$entryreturn =~ s/{{customnine}}/$gmcustomninetemplate/gi;
# # 		$entryreturn =~ s/{{customten}}/$gmcustomtentemplate/gi;
# 		$entryreturn =~ s/{{customone}}/$TEMPLATES->{'gmcustomonetemplate'}/gi;
# 		$entryreturn =~ s/{{customtwo}}/$TEMPLATES->{'gmcustomtwotemplate'}/gi;
# 		$entryreturn =~ s/{{customthree}}/$TEMPLATES->{'gmcustomthreetemplate'}/gi;
# 		$entryreturn =~ s/{{customfour}}/$TEMPLATES->{'gmcustomfourtemplate'}/gi;
# 		$entryreturn =~ s/{{customfive}}/$TEMPLATES->{'gmcustomfivetemplate'}/gi;
# 		$entryreturn =~ s/{{customsix}}/$TEMPLATES->{'gmcustomsixtemplate'}/gi;
# 		$entryreturn =~ s/{{customseven}}/$TEMPLATES->{'gmcustomseventemplate'}/gi;
# 		$entryreturn =~ s/{{customeight}}/$TEMPLATES->{'gmcustomeighttemplate'}/gi;
# 		$entryreturn =~ s/{{customnine}}/$TEMPLATES->{'gmcustomninetemplate'}/gi;
# 		$entryreturn =~ s/{{customten}}/$TEMPLATES->{'gmcustomtentemplate'}/gi;
# 	}
# 
# 	if( $entryreturn =~ m/{{logarchivelist}}/i ){
# 		if( $keepmonthlyarchives eq Gm_Constants::NO ){
# 			$entryreturn =~ s/{{logarchivelist}}//gi;
# 		} else {
# 			&gm_generatearchiveloglist;
# 			$entryreturn =~ s/{{logarchivelist}}/$logarchivelistfinal/gi;
# 		}
# 	}
# 
# if ($generateentrypages eq Gm_Constants::YES) {
# 	if (($entryreturn =~ m/{{logshortentrylist/i) || ($entryreturn =~ m/{{logmoreentrylist/i) || ($entryreturn =~ m/{{logentrylist/i)) {
# 		&gm_generateentryloglist;
# 		$entryreturn =~ s/{{logshortentrylist}}/$logshortentrylistfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist}}/$logmoreentrylistfinal/gi;
# 		$entryreturn =~ s/{{logentrylist}}/$logentrylistfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist month}}/$logshortentrylistmonthfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist day}}/$logshortentrylistdayfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist year}}/$logshortentrylistyearfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist month}}/$logmoreentrylistmonthfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist day}}/$logmoreentrylistdayfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist year}}/$logmoreentrylistyearfinal/gi;
# 		$entryreturn =~ s/{{logentrylist month}}/$logentrylistmonthfinal/gi;
# 		$entryreturn =~ s/{{logentrylist day}}/$logentrylistdayfinal/gi;
# 		$entryreturn =~ s/{{logentrylist year}}/$logentrylistyearfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist number}}/$logshortentrylistnumberfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist number}}/$logmoreentrylistnumberfinal/gi;
# 		$entryreturn =~ s/{{logentrylist number}}/$logentrylistnumberfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist firsthalf}}/$logshortentrylistfirsthalffinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist secondhalf}}/$logshortentrylistsecondhalffinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist firsthalf}}/$logmoreentrylistfirsthalffinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist secondhalf}}/$logmoreentrylistsecondhalffinal/gi;
# 		$entryreturn =~ s/{{logentrylist firsthalf}}/$logentrylistfirsthalffinal/gi;
# 		$entryreturn =~ s/{{logentrylist secondhalf}}/$logentrylistsecondhalffinal/gi;
# 	}
# 	if (($entryreturn =~ m/{{logshortentrylist comments/i) || ($entryreturn =~ m/{{logmoreentrylist comments/i) || ($entryreturn =~ m/{{logentrylist comments/i)) {
# 		&gm_generateentryloglistcomments;
# 		$entryreturn =~ s/{{logshortentrylist comments}}/$logshortentrylistfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist comments}}/$logmoreentrylistfinal/gi;
# 		$entryreturn =~ s/{{logentrylist comments}}/$logentrylistfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist commentsminimum}}/$logshortminimumentrylistfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist commentsminimum}}/$logmoreminimumentrylistfinal/gi;
# 		$entryreturn =~ s/{{logentrylist commentsminimum}}/$logminimumentrylistfinal/gi;
# 		$entryreturn =~ s/{{logshortentrylist commentsnumber}}/$logshortnumberentrylistfinal/gi;
# 		$entryreturn =~ s/{{logmoreentrylist commentsnumber}}/$logmorenumberentrylistfinal/gi;
# 		$entryreturn =~ s/{{logentrylist commentsnumber}}/$lognumberentrylistfinal/gi;
# 	}
# 	if (($entryreturn =~ m/{{logshortentrylist /i) || ($entryreturn =~ m/{{logmoreentrylist /i) || 
# 			($entryreturn =~ m/{{logentrylist /i)) {
# 
# 		my $gmauthors = Gm_Storage::getAuthors( errHandler=>\&Gm_Web::displayAdminErrorExit );
# 
# 		foreach my $author ( keys( %$gmauthors ) ) {
# 			$thisentryloglistauthor = $gmauthors->{$author}{'author'};
# 			&gm_generateentryloglistauthor;
# 			$entryreturn =~ s/{{logshortentrylist $thisentryloglistauthor}}/$logshortentrylistfinal/gi;
# 			$entryreturn =~ s/{{logmoreentrylist $thisentryloglistauthor}}/$logmoreentrylistfinal/gi;
# 			$entryreturn =~ s/{{logentrylist $thisentryloglistauthor}}/$logentrylistfinal/gi;
# 		}
# 	}
# }
# 
# if (($entryreturn =~ m/{{calendar}}/i) || ($entryreturn =~ m/{{calendarweek}}/i)) {
# 	$usethisentryweekdaynumber = $thisentryweekdaynumber;
# 	$usethisentryday = $thisentryday;
# 	$usethisentrymonth = $thisentrymonth;
# 	$usethisentrymonthmonth = $thisentrymonthmonth;
# 	$usethisentrymonthword = $thisentrymonthword;
# 	$usethisentryyear = $thisentryyear;
# 	$usethisentryyearyear = $thisentryyearyear;
# 	&gm_generatecalendar;
# 	$entryreturn =~ s/{{calendar}}/$calendarfull/gi;
# 	$entryreturn =~ s/{{calendarweek}}/$calendarweekfull/gi;
# }
# 
# if ($entryreturn =~ m/{{calendar (..)\/(..)}}/i) {
# 
# 	until ($entryreturn !~ m/{{calendar (..)\/(..)}}/ig) {
# 
# 		$usethisentrymonthmonth = $1;
# 		$usethisentryyear = $2;
# 
# 		my $gmentrylist = Gm_Storage::getEntrylist( errHandler=>\&Gm_Web::displayAdminErrorExit );
# 
# 		$gotthecalendarmonth = Gm_Constants::NO;
# 		foreach my $entry ( sort { $gmentrylist->{$b}{'id'} <=> $gmentrylist->{$a}{'id'} } keys( %$gmentrylist ) ) {
# 			$loglistnumber = $gmentrylist->{$entry}{'id'};
# 			$loglistauthor = $gmentrylist->{$entry}{'author'};
# 			$loglistsubject = $gmentrylist->{$entry}{'subject'};
# 			$loglistdate = $gmentrylist->{$entry}{'created'};
# 			$loglisttimeampm = $gmentrylist->{$entry}{'createt'};
# 			$loglistopenstatus = $gmentrylist->{$entry}{'status'};
# 			$loglistmorestatus = $gmentrylist->{$entry}{'extended'};
# 			$loglistentrymusic = $gmentrylist->{$entry}{'music'};
# 			$loglistentrymood = $gmentrylist->{$entry}{'mood'};
# 			$loglistemoticonstatus = $gmentrylist->{$entry}{'emoticons'};
# 			&gm_getloglistvariables;
# 			if (($loglistmonthmonth eq $usethisentrymonthmonth) && ($loglistyear eq $usethisentryyear) && ($gotthecalendarmonth eq Gm_Constants::NO)) {
# 				$usethisentryweekdaynumber = $loglistentryweekdaynumber;
# 				$usethisentryday = $loglistday;
# 				$usethisentrymonth = $loglistmonth;
# 				$usethisentrymonthword = $loglistmonthword;
# 				$usethisentryyearyear = $loglistyearyear;
# 				&gm_generatecalendar;
# 				$entryreturn =~ s/{{calendar $usethisentrymonthmonth\/$usethisentryyear}}/$calendarfull/gi;
# 				$gotthecalendarmonth = Gm_Constants::YES;
# 			}
# 		}
# 
# 		if ($entryreturn =~ m/{{calendar $usethisentrymonthmonth\/$usethisentryyear}}/i) {
# 			$entryreturn =~ s/{{calendar $usethisentrymonthmonth\/$usethisentryyear}}//gi;
# 		}
# 
# 	}
# 
# }
# 
# # emoticons can be off, so we provide {{smartemoticonscode}} to print them only when they are needed/wanted: linear 9/23/2003
# # merged in 1.3
# 
# 
# # merged 9/23/2003 
# 
# if (($entryreturn =~ m/{{commentdivider}}/i) || ($entryreturn =~ m/{{entrycommentsform}}/i) || ($entryreturn =~ m/{{entrykarmaform}}/i) || ($entryreturn =~ m/{{searchform}}/i)) {
# 	$entryreturn =~ s/{{commentdivider}}/$thisentrycommentdivider/gi;
# 	$entryreturn =~ s/{{entrycommentsform}}/$thisentrycommentsform/gi;
# 	$entryreturn =~ s/{{entrykarmaform}}/$thisentrykarmaform/gi;
# 	$entryreturn =~ s/{{searchform}}/$thisentrysearchform/gi;
# }
# 
# #merged 9/29/2003
# # added by flipped cracker
# # version 1.3
# ## autolinkentry functionality removed from 1.3.1. linear 2/19/2004 
# # merged in version 1.3
# 
# if ($entryreturn =~ m/{{entry/i) {
# 	$entryreturn =~ s/{{entrymainbody}}/$thisentrymainbody/gi;
# 	$entryreturn =~ s/{{entrymorebody}}/$thisentrymorebody/gi;
# 	$entryreturn =~ s/{{entrycomments}}/$thisentrycomments/gi;
# 	if ($entryreturn =~ m/{{entrymainbodyfirstwords (\d+)}}/i) {
# 		until ($entryreturn !~ m/{{entrymainbodyfirstwords (\d+)}}/isg) {
# 			$firstwordscount = $1;
# 			$grabmainbodywords = $thisentrymainbody;
# 			if ($grabmainbodywords =~ m/{{link/i) {
# 				$grabmainbodywords =~ s/({{linkmo) (http|https|ftp)(:\/\/\S+?) (.+?)(\|)(.+?)(}})/$4/isg;
# 				$grabmainbodywords =~ s/({{link) (http|https|ftp)(:\/\/\S+?)(}})/$2$3/isg;
# 				$grabmainbodywords =~ s/({{link) (http|https|ftp)(:\/\/\S+?) (.+?)(}})/$4/isg;
# ###########################################################
# # 9/2/2001 - CHERYL LAMBERT
# # ADDED {{linkblank}} TEMPLATE VARIABLE SO THAT I CAN USE target="_blank" TO OPEN NEW WINDOWS
#         $grabmainbodywords =~ s/({{linkblankmo) (http|https|ftp)(:\/\/\S+?) (.+?)(\|)(.+?)(}})/$4/isg;
#         $grabmainbodywords =~ s/({{linkblank) (http|https|ftp)(:\/\/\S+?)(}})/$2$3/isg;
#         $grabmainbodywords =~ s/({{linkblank) (http|https|ftp)(:\/\/\S+?) (.+?)(}})/$4/isg;
# ###########################################################				
# 			}
# 			if ($grabmainbodywords =~ m/{{email/i) {
# 				$grabmainbodywords =~ s/({{emailmo) (\S+\@\S+?) (.+?)(\|)(.+?)(}})/$3/isg;
# 				$grabmainbodywords =~ s/({{email) (\S+\@\S+?)(}})/$2/isg;
# 				$grabmainbodywords =~ s/({{email) (\S+\@\S+?) (.+?)(}})/$3/isg;
# 			}
# 			$grabmainbodywords =~ s/<([^>]|\n)*>/ /g;
# 			$grabmainbodywords =~ s/{{(.*?)}}/ /g;
# 			$grabmainbodywords =~ s/\n/ /g;
# 			$grabmainbodywords =~ s/\r/ /g;
# 			$grabmainbodywords =~ s/\|\*\|/ /g;
# 			$grabmainbodywords =~ s/^\s+//;
# 			$grabmainbodywords =~ s/\s+$//;
# 			$grabmainbodywords =~ s/\s{2,}/ /g;
# 			@grabmainbodywordslist = split (/ /, $grabmainbodywords);
# 			$countwordsfromhere = 0;
# 			(@finalmainbodywordslist, @finalmainbodywordslist = ());
# 			if ($firstwordscount < 1) { $firstwordscount = 1; }
# 			do {
# 				$finalmainbodywordslist[$countwordsfromhere] = $grabmainbodywordslist[$countwordsfromhere];
# 				$countwordsfromhere++;
# 			} until $countwordsfromhere eq $firstwordscount;
# 			$finalmainbodyfirstwords = join (" ", @finalmainbodywordslist);
# 			$finalmainbodyfirstwords =~ s/^\s+//;
# 			$finalmainbodyfirstwords =~ s/\s+$//;
# 			$finalmainbodyfirstwords =~ s/\s{2,}//g;
# 			if (substr($finalmainbodyfirstwords, -1) =~ /\W/) { chop($finalmainbodyfirstwords); }
# 			if (substr($finalmainbodyfirstwords, -1) eq / /) { chop($finalmainbodyfirstwords); }
# 			$entryreturn =~ s/{{entrymainbodyfirstwords ($firstwordscount)}}/$finalmainbodyfirstwords/isg;
# 		}
# 	}
# 	if ($entryreturn =~ m/{{entrymorebodyfirstwords (\d+)}}/i) {
# 		until ($entryreturn !~ m/{{entrymorebodyfirstwords (\d+)}}/isg) {
# 			$firstwordscount = $1;
# 			if ($thisentrymorebody eq Gm_Constants::EMPTY) {
# 				$entryreturn =~ s/{{entrymorebodyfirstwords ($firstwordscount)}}//isg;
# 			} else {
# 				$grabmorebodywords = $thisentrymorebody;
# 				if ($grabmorebodywords =~ m/{{link/i) {
# 					$grabmorebodywords =~ s/({{linkmo) (http|https|ftp)(:\/\/\S+?) (.+?)(\|)(.+?)(}})/$4/isg;
# 					$grabmorebodywords =~ s/({{link) (http|https|ftp)(:\/\/\S+?)(}})/$2$3/isg;
# 					$grabmorebodywords =~ s/({{link) (http|https|ftp)(:\/\/\S+?) (.+?)(}})/$4/isg;
# ###########################################################
# # 9/2/2001 - CHERYL LAMBERT
# # ADDED {{linkblank}} TEMPLATE VARIABLE SO THAT I CAN USE target="_blank" TO OPEN NEW WINDOWS
#           $grabmorebodywords =~ s/({{linkblankmo) (http|https|ftp)(:\/\/\S+?) (.+?)(\|)(.+?)(}})/$4/isg;
#           $grabmorebodywords =~ s/({{linkblank) (http|https|ftp)(:\/\/\S+?)(}})/$2$3/isg;
#           $grabmorebodywords =~ s/({{linkblank) (http|https|ftp)(:\/\/\S+?) (.+?)(}})/$4/isg;
# ###########################################################					
# 				}
# 				if ($grabmorebodywords =~ m/{{email/i) {
# 					$grabmorebodywords =~ s/({{emailmo) (\S+\@\S+?) (.+?)(\|)(.+?)(}})/$3/isg;
# 					$grabmorebodywords =~ s/({{email) (\S+\@\S+?)(}})/$2/isg;
# 					$grabmorebodywords =~ s/({{email) (\S+\@\S+?) (.+?)(}})/$3/isg;
# 				}
# 				$grabmorebodywords =~ s/<([^>]|\n)*>/ /g;
# 				$grabmorebodywords =~ s/{{(.*?)}}/ /g;
# 				$grabmorebodywords =~ s/\n/ /g;
# 				$grabmorebodywords =~ s/\r/ /g;
# 				$grabmorebodywords =~ s/\|\*\|/ /g;
# 				$grabmorebodywords =~ s/^\s+//;
# 				$grabmorebodywords =~ s/\s+$//;
# 				$grabmorebodywords =~ s/\s{2,}/ /g;
# 				@grabmorebodywordslist = split (/ /, $grabmorebodywords);
# 				$countwordsfromhere = 0;
# 				(@finalmorebodywordslist, @finalmorebodywordslist = ());
# 				if ($firstwordscount < 1) { $firstwordscount = 1; }
# 				do {
# 					$finalmorebodywordslist[$countwordsfromhere] = $grabmorebodywordslist[$countwordsfromhere];
# 					$countwordsfromhere++;
# 				} until $countwordsfromhere eq $firstwordscount;
# 				$finalmorebodyfirstwords = join (" ", @finalmorebodywordslist);
# 				$finalmorebodyfirstwords =~ s/^\s+//;
# 				$finalmorebodyfirstwords =~ s/\s+$//;
# 				$finalmorebodyfirstwords =~ s/\s{2,}//g;
# 				if (substr($finalmorebodyfirstwords, -1) =~ /\W/) { chop($finalmorebodyfirstwords); }
# 				if (substr($finalmorebodyfirstwords, -1) eq / /) { chop($finalmorebodyfirstwords); }
# 				$entryreturn =~ s/{{entrymorebodyfirstwords ($firstwordscount)}}/$finalmorebodyfirstwords/isg;
# 			}
# 		}
# 	}
# }
# 
# 
# if (($entryreturn =~ m/link}}/i) || ($entryreturn =~ m/{{morepreface}}/i)) {
# 	$entryreturn =~ s/{{morepreface}}/$thisentrymorepreface/gi;
# 	$entryreturn =~ s/{{morelink}}/$thisentrymorelink/gi;
# 	$entryreturn =~ s/{{karmalink}}/$thisentrykarmalink/gi;
# 	$entryreturn =~ s/{{commentslink}}/$thisentrycommentslink/gi;
# 	$entryreturn =~ s/{{pagelink}}/$thisentrypagelink/gi;
# 	$entryreturn =~ s/{{pageindexlink}}/$thisentrypageindexlink/gi;
# 	$entryreturn =~ s/{{pagearchiveindexlink}}/$thisentrypagearchiveindexlink/gi;
# 	$entryreturn =~ s/{{pagearchivelogindexlink}}/$thisentrypagearchivelogindexlink/gi;
# 	$entryreturn =~ s/{{pagesmartindexlink}}/$thisentrypagesmartindexlink/gi;
# 	$entryreturn =~ s/{{commentspostlink}}/$thisentrycommentspostlink/gi;
# 	$entryreturn =~ s/{{authorsmartlink}}/$thisentryauthorsmartlink/gi;
# }
# 
# if (($entryreturn =~ m/karma}}/i) || ($entryreturn =~ m/karmalink}}/i)) {
# 	$entryreturn =~ s/{{positivekarmalink}}/$thisentrypositivekarmalink/gi;
# 	$entryreturn =~ s/{{negativekarmalink}}/$thisentrynegativekarmalink/gi;
# 	$entryreturn =~ s/{{positivekarma}}/$thisentrypositivekarma/gi;
# 	$entryreturn =~ s/{{negativekarma}}/$thisentrynegativekarma/gi;
# 	$entryreturn =~ s/{{totalkarma}}/$thisentrytotalkarma/gi;
# }
# 
# # added in 1.3.1 : flipped cracker 1/18/2005
# if ($entryreturn =~ m/{{smart/i) {
# 	# smartentrymusic and smartentrymood templates: linear 9/20/2003 
# 	# merged in 1.3
# 	if ($thisentrymusic ne Gm_Constants::EMPTY) {
# 	    $entryreturn =~ s/{{smartentrymusic}}/$gmsmartentrymusictemplate/gi;
# 	} else {
# 	    	$entryreturn =~ s/{{smartentrymusic}}//gi;
#     }
# 	if ($thisentrymood ne Gm_Constants::EMPTY) {
# 	    $entryreturn =~ s/{{smartentrymood}}/$gmsmartentrymoodtemplate/gi;
# 	} else {
# 	    	$entryreturn =~ s/{{smartentrymood}}//gi;
#     }
# 	# merged 9/20/2003
# }
# # added 1/18/2005
# if ($entryreturn =~ m/{{entry/i) {
# 	$entryreturn =~ s/{{entrysubject}}/$thisentrysubject/gi;
# 	###########################################################
# 	# 9/3/2001 - CHERYL LAMBERT
# 	# ADDED 1 TEMPLATE VARIABLE SO THAT I CAN STRIP THE HTML CODE FROM THE ENTRYSUBJECT
# 	# WHEN IT IS USED IN THE <TITLE> TAG
# 		my $thisentrysubjectstripped = $thisentrysubject;
# 		$thisentrysubjectstripped =~ s/<.*?>//gi;
# 		$thisentrysubjectstripped =~ s/<\/.*?>//gi;
# 		$entryreturn =~ s/{{entrysubjectstripped}}/$thisentrysubjectstripped/gi;
# 	###########################################################
# 	$entryreturn =~ s/{{entrynumber}}/$thisentrynumber/gi;
# 	$entryreturn =~ s/{{entrynumberpadded}}/$thisentrynumberpadded/gi;
# 
# 	# added by flipped cracker 09/12/03
# 	# version 1.3
# 	$entryreturn =~ s/{{entrymusic}}/$thisentrymusic/gi;
# 	$thisentrymood = &gm_emoticons($thisentrymood);
# 	$entryreturn =~ s/{{entrymood}}/$thisentrymood/gi;
# 	# end additions
# }
# if ($entryreturn =~ m/{{author/i) {
# 	$entryreturn =~ s/{{author}}/$thisentryauthor/gi;
# 	$entryreturn =~ s/{{authoremail}}/$thisentryauthoremail/gi;
# 	$entryreturn =~ s/{{authorhomepage}}/$thisentryauthorhomepage/gi;
# 	$entryreturn =~ s/{{authorentrycount}}/$thisentryauthorentrycount/gi;
# }
# 
# if (($entryreturn =~ m/{{authoremail /i) || ($entryreturn =~ m/{{authorhomepage /i) || ($entryreturn =~ m/{{authorentrycount /i) || ($entryreturn =~ m/{{authoremaillink /i) || ($entryreturn =~ m/{{authorhomepagelink /i) || ($entryreturn =~ m/{{authorsmartlink /i)) {
# 	&gm_generateexternalauthorvariables;
# }
# 
# if ($entryreturn =~ m/{{comments/i) {
# 	$entryreturn =~ s/{{commentstatussmart}}/$thisentrycommentstatussmart/gi;
# 	$entryreturn =~ s/{{commentstatussmartupper}}/$thisentrycommentstatussmartupper/gi;
# 	$entryreturn =~ s/{{commentstatussmartlower}}/$thisentrycommentstatussmartlower/gi;
# 	$entryreturn =~ s/{{commentsnumber}}/$thisentrycommentsnumber/gi;
# }
# 
# if (($entryreturn =~ m/{{day/i) || ($entryreturn =~ m/{{month/i) || ($entryreturn =~ m/{{year/i) || ($entryreturn =~ m/{{hour/i) || ($entryreturn =~ m/{{minute/i) || ($entryreturn =~ m/{{second/i) || ($entryreturn =~ m/{{weekday/i) || ($entryreturn =~ m/{{militaryhour}}/i) || ($entryreturn =~ m/{{ampm/i) || ($entryreturn =~ m/{{timezone}}/i)) {
# 	$entryreturn =~ s/{{day}}/$thisentryday/gi;
# 	$entryreturn =~ s/{{month}}/$thisentrymonth/gi;
# 	$entryreturn =~ s/{{year}}/$thisentryyear/gi;
# 	$entryreturn =~ s/{{hour}}/$thisentryhour/gi;
# 	$entryreturn =~ s/{{minute}}/$thisentryminute/gi;
# 	$entryreturn =~ s/{{second}}/$thisentrysecond/gi;
# 	$entryreturn =~ s/{{dayday}}/$thisentrydayday/gi;
# 	$entryreturn =~ s/{{monthmonth}}/$thisentrymonthmonth/gi;
# 	$entryreturn =~ s/{{yearyear}}/$thisentryyearyear/gi;
# 	$entryreturn =~ s/{{hourhour}}/$thisentryhourhour/gi;
# 	$entryreturn =~ s/{{minuteminute}}/$thisentryminuteminute/gi;
# 	$entryreturn =~ s/{{secondsecond}}/$thisentrysecondsecond/gi;
# 	$entryreturn =~ s/{{weekday}}/$thisentryweekday/gi;
# 	$entryreturn =~ s/{{weekdayshort}}/$thisentryweekdayshort/gi;	
# 	$entryreturn =~ s/{{monthword}}/$thisentrymonthword/gi;
# 	$entryreturn =~ s/{{monthwordshort}}/$thisentrymonthwordshort/gi;	
# 	$entryreturn =~ s/{{weekdayupper}}/$thisentryweekdayupper/gi;
# 	$entryreturn =~ s/{{monthwordupper}}/$thisentrymonthwordupper/gi;
# 	$entryreturn =~ s/{{weekdaylower}}/$thisentryweekdaylower/gi;
# 	$entryreturn =~ s/{{monthwordlower}}/$thisentrymonthwordlower/gi;
# 	$entryreturn =~ s/{{weekdayuppershort}}/$thisentryweekdayuppershort/gi;
# 	$entryreturn =~ s/{{monthworduppershort}}/$thisentrymonthworduppershort/gi;
# 	$entryreturn =~ s/{{weekdaylowershort}}/$thisentryweekdaylowershort/gi;
# 	$entryreturn =~ s/{{monthwordlowershort}}/$thisentrymonthwordlowershort/gi;
# 	$entryreturn =~ s/{{militaryhour}}/$thisentrymilitaryhour/gi;
# 	$entryreturn =~ s/{{ampm}}/$thisentryampm/gi;
# 	$entryreturn =~ s/{{ampmdot}}/$thisentryampmdot/gi;
# 	$entryreturn =~ s/{{ampmlower}}/$thisentryampmlower/gi;
# 	$entryreturn =~ s/{{ampmdotlower}}/$thisentryampmdotlower/gi;
# 	$entryreturn =~ s/{{timezone}}/$timezone/gi;
#     # new  ordinal numbers for {{dayappend}} variable: linear 9/15/2003 
#     # merged in 1.3
#     $thisentrydayappend = Gm_Utils::toOrdinal($thisentryday); 
#     #merged 9/15/2003
#     # new {{dayappend}} variable:: linear 9/15/2003 
#     # merged in 1.3
#    	$thisentrydayappend = Gm_Utils::toOrdinal($thisentryday);
#     $entryreturn =~ s/{{dayappend}}/$thisentrydayappend/gi;
#     # merged 9/15/2003 
# }
# 
# if (($entryreturn =~ m/{{weekbeginning/i) || ($entryreturn =~ m/{{weekending/i)) {
# 	$entryreturn =~ s/{{weekbeginningday}}/$thisentryweekbeginningday/gi;
# 	$entryreturn =~ s/{{weekbeginningdayday}}/$thisentryweekbeginningdayday/gi;
# 	$entryreturn =~ s/{{weekbeginningmonth}}/$thisentryweekbeginningmonth/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthmonth}}/$thisentryweekbeginningmonthmonth/gi;
# 	$entryreturn =~ s/{{weekbeginningyear}}/$thisentryweekbeginningyear/gi;
# 	$entryreturn =~ s/{{weekbeginningyearyear}}/$thisentryweekbeginningyearyear/gi;
# 	$entryreturn =~ s/{{weekbeginningweekday}}/$thisentryweekbeginningweekday/gi;
# 	$entryreturn =~ s/{{weekbeginningweekdayshort}}/$thisentryweekbeginningweekdayshort/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthword}}/$thisentryweekbeginningmonthword/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthwordshort}}/$thisentryweekbeginningmonthwordshort/gi;
# 	$entryreturn =~ s/{{weekbeginningweekdayupper}}/$thisentryweekbeginningweekdayupper/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthwordupper}}/$thisentryweekbeginningmonthwordupper/gi;
# 	$entryreturn =~ s/{{weekbeginningweekdaylower}}/$thisentryweekbeginningweekdaylower/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthwordlower}}/$thisentryweekbeginningmonthwordlower/gi;
# 	$entryreturn =~ s/{{weekbeginningweekdayuppershort}}/$thisentryweekbeginningweekdayuppershort/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthworduppershort}}/$thisentryweekbeginningmonthworduppershort/gi;
# 	$entryreturn =~ s/{{weekbeginningweekdaylowershort}}/$thisentryweekbeginningweekdaylowershort/gi;
# 	$entryreturn =~ s/{{weekbeginningmonthwordlowershort}}/$thisentryweekbeginningmonthwordlowershort/gi;
# 	$entryreturn =~ s/{{weekendingday}}/$thisentryweekendingday/gi;
# 	$entryreturn =~ s/{{weekendingdayday}}/$thisentryweekendingdayday/gi;
# 	$entryreturn =~ s/{{weekendingmonth}}/$thisentryweekendingmonth/gi;
# 	$entryreturn =~ s/{{weekendingmonthmonth}}/$thisentryweekendingmonthmonth/gi;
# 	$entryreturn =~ s/{{weekendingyear}}/$thisentryweekendingyear/gi;
# 	$entryreturn =~ s/{{weekendingyearyear}}/$thisentryweekendingyearyear/gi;
# 	$entryreturn =~ s/{{weekendingweekday}}/$thisentryweekendingweekday/gi;
# 	$entryreturn =~ s/{{weekendingweekdayshort}}/$thisentryweekendingweekdayshort/gi;	
# 	$entryreturn =~ s/{{weekendingmonthword}}/$thisentryweekendingmonthword/gi;
# 	$entryreturn =~ s/{{weekendingmonthwordshort}}/$thisentryweekendingmonthwordshort/gi;	
# 	$entryreturn =~ s/{{weekendingweekdayupper}}/$thisentryweekendingweekdayupper/gi;
# 	$entryreturn =~ s/{{weekendingmonthwordupper}}/$thisentryweekendingmonthwordupper/gi;
# 	$entryreturn =~ s/{{weekendingweekdaylower}}/$thisentryweekendingweekdaylower/gi;
# 	$entryreturn =~ s/{{weekendingmonthwordlower}}/$thisentryweekendingmonthwordlower/gi;
# 	$entryreturn =~ s/{{weekendingweekdayuppershort}}/$thisentryweekendingweekdayuppershort/gi;
# 	$entryreturn =~ s/{{weekendingmonthworduppershort}}/$thisentryweekendingmonthworduppershort/gi;
# 	$entryreturn =~ s/{{weekendingweekdaylowershort}}/$thisentryweekendingweekdaylowershort/gi;
# 	$entryreturn =~ s/{{weekendingmonthwordlowershort}}/$thisentryweekendingmonthwordlowershort/gi;
# }
# 
# if ($entryreturn =~ m/{{link/i) {
# 	$entryreturn =~ s/({{linkmo) (http|https|ftp)(:\/\/\S+?) (.+?)(\|)(.+?)(}})/<a href="$2$3" onmouseover="window.status='$6';return true" onmouseout="window.status='';return true">$4<\/a>/isg;
# 	$entryreturn =~ s/({{link) (http|https|ftp)(:\/\/\S+?)(}})/<a href="$2$3">$2$3<\/a>/isg;
# 	$entryreturn =~ s/({{link) (http|https|ftp)(:\/\/\S+?) (.+?)(}})/<a href="$2$3">$4<\/a>/isg;
# 	###########################################################
# 	# 9/2/2001 - CHERYL LAMBERT
# 	# ADDED {{linkblank}} TEMPLATE VARIABLE SO THAT I CAN USE target="_blank" TO OPEN NEW WINDOWS
# 	$entryreturn =~ s/({{linkblankmo) (http|https|ftp)(:\/\/\S+?) (.+?)(\|)(.+?)(}})/<A HREF="$2$3" onMouseOver="window.status='$6';return true" onMouseOut="window.status='';return true" target="_blank">$4<\/A>/isg;
# 	$entryreturn =~ s/({{linkblank) (http|https|ftp)(:\/\/\S+?)(}})/<A HREF="$2$3" target="_blank">$2$3<\/A>/isg;
# 	$entryreturn =~ s/({{linkblank) (http|https|ftp)(:\/\/\S+?) (.+?)(}})/<A HREF="$2$3" target="_blank">$4<\/A>/isg;
# 	###########################################################
# }
# 
# if ($entryreturn =~ m/{{email/i) {
# 	$entryreturn =~ s/({{emailmo) (\S+\@\S+?) (.+?)(\|)(.+?)(}})/<a href="mailto:$2" onmouseover="window.status='$5';return true" onmouseout="window.status='';return true">$3<\/>/isg;
# 	$entryreturn =~ s/({{email) (\S+\@\S+?)(}})/<a href="mailto:$2">$2<\/a>/isg;
# 	$entryreturn =~ s/({{email) (\S+\@\S+?) (.+?)(}})/<a href="mailto:$2">$3<\/a>/isg;
# }
# 
# if ($entryreturn =~ m/{{popup (\S+) (.+?) (\d+)x(\d+)}}/i) {
# 	until ($entryreturn !~ m/{{popup (\S+) (.+?) (\d+)x(\d+)}}/isg) {
# 
# 		$popupfile = $1;
# 		$popuptitle = $2;
# 		$popupwidth = $3;
# 		$popupheight = $4;
# 		$popuphtmlfile = $1;
# 		$popuphtmlfile =~ s/\.(\S+)$//;
# 		$popuphtmlfile .= ".$entrysuffix";
# 		$popuppage = $gmpopuppagetemplate;
# 		$popupcode = $gmpopupcodetemplate;
# 		$popuppage =~ s/{{popupfile}}/$popupfile/gi;
# 		$popuppage =~ s/{{popuphtmlfile}}/$popuphtmlfile/gi;
# 		$popuppage =~ s/{{popuptitle}}/$popuptitle/gi;
# 		$popuppage =~ s/{{popupwidth}}/$popupwidth/gi;
# 		$popuppage =~ s/{{popupheight}}/$popupheight/gi;
# 		$popupcode =~ s/{{popupfile}}/$popupfile/gi;
# 		$popupcode =~ s/{{popuphtmlfile}}/$popuphtmlfile/gi;
# 		$popupcode =~ s/{{popuptitle}}/$popuptitle/gi;
# 		$popupcode =~ s/{{popupwidth}}/$popupwidth/gi;
# 		$popupcode =~ s/{{popupheight}}/$popupheight/gi;
# 		$popuppage =~ s/{{cgiwebpath}}/$cgiwebpath/gi;
# 		$popuppage =~ s/{{entrieswebpath}}/$EntriesWebPath/gi;
# 		$popuppage =~ s/{{logwebpath}}/$LogWebPath/gi;
# 		$popuppage =~ s/{{emoticonspath}}/$emoticonspath/gi;
# 
# 		$foundpopupmatch = Gm_Constants::NO;
# 		if( exists( $gmentrylist->{$thisentrynumber} ) ) {
# 			$loglistnumber = $gmentrylist->{$thisentrynumber}{'id'};
# 			$loglistauthor = $gmentrylist->{$thisentrynumber}{'author'};
# 			$loglistsubject = $gmentrylist->{$thisentrynumber}{'subject'};
# 			$loglistdate = $gmentrylist->{$thisentrynumber}{'created'};
# 			$loglisttimeampm = $gmentrylist->{$thisentrynumber}{'createt'};
# 			$loglistopenstatus = $gmentrylist->{$thisentrynumber}{'status'};
# 			$loglistmorestatus = $gmentrylist->{$thisentrynumber}{'extended'};
# 			&gm_getloglistvariables;
# 			$foundpopupmatch = Gm_Constants::YES;
# 		}
# 
# 		$listsubsub = $popuppage;
# 		&entrylistsubsub;
# 
# 		open (POPUPVIDEO, ">$EntriesPath/$popuphtmlfile") || &gm_dangermouse("Can't write to $EntriesPath/$popuphtmlfile.  Please make sure your entries/archives directory is correctly set and is CHMODed to 777; also, try running Diagnostics & Repair in the Configuration screen.");
# 		print POPUPVIDEO $listsubsub;
# 		close (POPUPVIDEO);
# 
# 		$listsubsub = $popupcode;
# 		&entrylistsubsub;
# 
# 		$entryreturn =~ s/{{popup $popupfile $popuptitle ($popupwidth)x($popupheight)}}/$listsubsub/isg;
# 
# 	}
# }
# 
# if (($entryreturn =~ m/{{previous/i) || ($entryreturn =~ m/{{next/i)) {
# 
# 	$entryreturn =~ s/{{previousmore/{{moreprevious/isg;
# 	$entryreturn =~ s/{{nextmore/{{morenext/isg;
# 
# 	if ($thisentrynumber ne "1") {
# 		$thispreviousentrynumber = $thisentrynumber - 1;
# 		$foundregular = Gm_Constants::NO;
# 		$foundmore = Gm_Constants::NO;
# 		$foundboth = Gm_Constants::NO;
# 		do {
# 			foreach my $entry ( sort { $gmentrylist->{$b}{'id'} <=> $gmentrylist->{$a}{'id'} } keys( %$gmentrylist ) ) {
# 				$loglistnumber = $gmentrylist->{$entry}{'id'};
# 				$loglistauthor = $gmentrylist->{$entry}{'author'};
# 				$loglistsubject = $gmentrylist->{$entry}{'subject'};
# 				$loglistdate = $gmentrylist->{$entry}{'created'};
# 				$loglisttimeampm = $gmentrylist->{$entry}{'createt'};
# 				$loglistopenstatus = $gmentrylist->{$entry}{'status'};
# 				$loglistmorestatus = $gmentrylist->{$entry}{'extended'};
# 
# 				if (($thispreviousentrynumber eq $loglistnumber) && ($loglistnumber < $thisentrynumber) && ($foundregular eq Gm_Constants::NO) && ($loglistopenstatus ne "C")) {
# 					$entryreturn =~ s/{{previouslink}}/$gmpreviouslinktemplate/isg;
# 					&gm_getloglistvariables;
# 					$listsubsub = $entryreturn;
# 					$listsubsub =~ s/{{previous/{{/isg;
# 					&entrylistsubsub;
# 					$entryreturn = $listsubsub;
# 					$foundregular = Gm_Constants::YES;
# 				}
# 				if (($thispreviousentrynumber eq $loglistnumber) && ($loglistmorestatus eq 'Y') && ($loglistnumber < $thisentrynumber) && ($foundmore eq Gm_Constants::NO) && ($loglistopenstatus ne "C")) {
# 					$entryreturn =~ s/{{morepreviouslink}}/$gmpreviousmorelinktemplate/isg;
# 					$entryreturn =~ s/{{previousmore/{{moreprevious/isg;
# 					&gm_getloglistvariables;
# 					$listsubsub = $entryreturn;
# 					$listsubsub =~ s/{{moreprevious/{{/isg;
# 					&entrylistsubsub;
# 					$entryreturn = $listsubsub;
# 					$foundmore = Gm_Constants::YES;
# 				}
# 			}
# 
# 			if (($foundregular eq Gm_Constants::YES) && ($foundmore eq Gm_Constants::YES)) { $foundboth = Gm_Constants::YES; }
# 			if ($thispreviousentrynumber eq "1") { $foundboth = Gm_Constants::YES; }
# 			$thispreviousentrynumber--;
# 
# 		} until $foundboth eq Gm_Constants::YES;
# 	}
# 
# 	if ($thisentrynumber ne $newentrynumber) {
# 		$thisnextentrynumber = $thisentrynumber + 1;
# 		$foundregular = Gm_Constants::NO;
# 		$foundmore = Gm_Constants::NO;
# 		$foundboth = Gm_Constants::NO;
# 		do {
# 			foreach my $entry ( sort { $gmentrylist->{$a}{'id'} <=> $gmentrylist->{$b}{'id'} } keys( %$gmentrylist ) ) {
# 				$loglistnumber = $gmentrylist->{$entry}{'id'};
# 				$loglistauthor = $gmentrylist->{$entry}{'author'};
# 				$loglistsubject = $gmentrylist->{$entry}{'subject'};
# 				$loglistdate = $gmentrylist->{$entry}{'created'};
# 				$loglisttimeampm = $gmentrylist->{$entry}{'createt'};
# 				$loglistopenstatus = $gmentrylist->{$entry}{'status'};
# 				$loglistmorestatus = $gmentrylist->{$entry}{'extended'};
# 
# 				if (($thisnextentrynumber eq $loglistnumber) && ($loglistnumber > $thisentrynumber) && ($foundregular eq Gm_Constants::NO) && ($loglistopenstatus ne "C")) {
# 					$entryreturn =~ s/{{nextlink}}/$gmnextlinktemplate/isg;
# 					&gm_getloglistvariables;
# 					$listsubsub = $entryreturn;
# 					$listsubsub =~ s/{{next/{{/isg;
# 					&entrylistsubsub;
# 					$entryreturn = $listsubsub;
# 					$foundregular = Gm_Constants::YES;
# 				}
# 				if (($thisnextentrynumber eq $loglistnumber) && ($loglistmorestatus eq 'Y') && ($loglistnumber > $thisentrynumber) && ($foundmore eq Gm_Constants::NO) && ($loglistopenstatus ne "C")) {
# 					$entryreturn =~ s/{{morenextlink}}/$gmnextmorelinktemplate/isg;
# 					$entryreturn =~ s/{{nextmore/{{morenext/isg;
# 					&gm_getloglistvariables;
# 					$listsubsub = $entryreturn;
# 					$listsubsub =~ s/{{morenext/{{/isg;
# 					&entrylistsubsub;
# 					$entryreturn = $listsubsub;
# 					$foundmore = Gm_Constants::YES;
# 				}
# 			}
# 
# 			if (($foundregular eq Gm_Constants::YES) && ($foundmore eq Gm_Constants::YES)) { $foundboth = Gm_Constants::YES; }
# 			if ($thisnextentrynumber eq $newentrynumber) { $foundboth = Gm_Constants::YES; }
# 			$thisnextentrynumber++;
# 
# 		} until $foundboth eq Gm_Constants::YES;
# 	}
# 
# 	$entryreturn =~ s/{{previouspagelink}}/$thisentrypagesmartindexlink/isg;
# 	$entryreturn =~ s/{{nextpagelink}}/$thisentrypagesmartindexlink/isg;
# 	$entryreturn =~ s/{{previousmorepagelink}}/$thisentrypagesmartindexlink/isg;
# 	$entryreturn =~ s/{{nextmorepagelink}}/$thisentrypagesmartindexlink/isg;
# 	$entryreturn =~ s/{{morepreviouspagelink}}/$thisentrypagesmartindexlink/isg;
# 	$entryreturn =~ s/{{morenextpagelink}}/$thisentrypagesmartindexlink/isg;
# 	$entryreturn =~ s/{{previouslink}}//isg;
# 	$entryreturn =~ s/{{nextlink}}//isg;
# 
# 	if (($entryreturn =~ m/{{previous/i) || ($entryreturn =~ m/{{next/i)) {
# 		if( exists( $gmentrylist->{$thisentrynumber} ) ) {
# 			$loglistnumber = $gmentrylist->{$thisentrynumber}{'id'};
# 			$loglistauthor = $gmentrylist->{$thisentrynumber}{'author'};
# 			$loglistsubject = $gmentrylist->{$thisentrynumber}{'subject'};
# 			$loglistdate = $gmentrylist->{$thisentrynumber}{'created'};
# 			$loglisttimeampm = $gmentrylist->{$thisentrynumber}{'createt'};
# 			$loglistopenstatus = $gmentrylist->{$thisentrynumber}{'status'};
# 			$loglistmorestatus = $gmentrylist->{$thisentrynumber}{'extended'};
# 
# 			&gm_getloglistvariables;
# 			$listsubsub = $entryreturn;
# 			$listsubsub =~ s/{{previous/{{/isg;
# 			$listsubsub =~ s/{{next/{{/isg;
# 			&entrylistsubsub;
# 			$entryreturn = $listsubsub;
# 		}
# 	}
# 
# 	$entryreturn =~ s/{{morepagelink}}/$thisentrypagesmartindexlink/isg;
# 
# }
# 
# 
# # enhanced escaping of metacharacters: linear 8/25/2003
# # merged in 1.3 
# if ($entryreturn =~ m/window.status='(.*?)';/i) { 
#    $entryreturn =~ s/\(/GMLEFTPARENTHESES/g; 
#    $entryreturn =~ s/\)/GMRIGHTPARENTHESES/g; 
#    $entryreturn =~ s/\?/GMQUESTIONMARK/g; 
#    $entryreturn =~ s/\+/GMPLUS/g; 
#    $entryreturn =~ s/\[/GMLEFTSQUAREBRACKET/g; 
#    $entryreturn =~ s/\]/GMRIGHTSQUAREBRACKET/g; 
#    $entryreturn =~ s/\$/GMDOLLAR/g; 
#    $entryreturn =~ s/\^/GMCARET/g; 
#    $entryreturn =~ s/\|/GMPIPE/g; 
#    $entryreturn =~ s/\*/GMSPLAT/g; 
#    until ($entryreturn !~ m/window.status='(.*?)';/ig) { 
#       $windowstatusorigstring = $1; 
#       $windowstatusmiddlenew = $1; 
#       $windowstatusmiddlenew =~ s#'#\\'#isg; 
#       $windowstatusmiddlenew =~ s#"#\\'#isg; 
#       $entryreturn =~ s/window.status='$windowstatusorigstring';/WSPLACEHOLDER='$windowstatusmiddlenew';/isg; 
#    } 
#    $entryreturn =~ s/WSPLACEHOLDER=/window.status=/isg; 
#    $entryreturn =~ s/GMLEFTPARENTHESES/\(/g; 
#    $entryreturn =~ s/GMRIGHTPARENTHESES/\)/g; 
#    $entryreturn =~ s/GMQUESTIONMARK/\?/g; 
#    $entryreturn =~ s/GMPLUS/\+/g; 
#    $entryreturn =~ s/GMLEFTSQUAREBRACKET/\[/g; 
#    $entryreturn =~ s/GMRIGHTSQUAREBRACKET/\]/g; 
#    $entryreturn =~ s/GMDOLLAR/\$/g; 
#    $entryreturn =~ s/GMCARET/\^/g; 
#    $entryreturn =~ s/GMPIPE/\|/g; 
#    $entryreturn =~ s/GMSPLAT/\*/g; 
# 
# }
# 
# if ($entryreturn =~ m/{{randomnumber (\d+)-(\d+)}}/i) {
# 	until ($entryreturn !~ m/{{randomnumber (\d+)-(\d+)}}/isg) {
# 		$minrand = $1;
# 		$maxrand = $2;
# 		$maxtemprand = $maxrand - $minrand;
# 		$maxtemprand++;
# 		$randresult = int(rand $maxtemprand) + $minrand;
# 		$entryreturn =~ s/{{randomnumber ($minrand)-($maxrand)}}/$randresult/i;
# 	}
# }
# 
# if (($entryreturn =~ m/{{cgiwebpath}}/i) || ($entryreturn =~ m/{{entrieswebpath}}/i) || ($entryreturn =~ m/{{logwebpath}}/i) || ($entryreturn =~ m/{{emoticonspath}}/i)) {
# 	$entryreturn =~ s/{{cgiwebpath}}/$cgiwebpath/gi;
# 	$entryreturn =~ s/{{entrieswebpath}}/$EntriesWebPath/gi;
# 	$entryreturn =~ s/{{logwebpath}}/$LogWebPath/gi;
# 	$entryreturn =~ s/{{emoticonspath}}/$emoticonspath/gi;
# }
# 
# if (($entryreturn =~ m/{{gmversion}}/i) || ($entryreturn =~ m/{{gmicon}}/i)) {
# 	$entryreturn =~ s/{{gmversion}}/$gmversion/gi;
# 	$entryreturn =~ s/{{gmicon}}/<a href="http:\/\/greymatterforum.proboards82.com\/" target="_top"><img border="0" src="$LogWebPath\/gm-icon.gif" alt="Powered By Greymatter" \/><\/a>/gi;
# }
# # new {{cookiescode}} template to facilitate setting cookies on commenters: linear 9/23/2003 
# # merged in 1.3
# $entryreturn =~ s/{{cookiescode}}/$gmcookiescodetemplate/gi;
# # merged 9/23/2003 
# # added by flipped cracker 09/12/03
# # version 1.3
# $entryreturn =~ s/{{emoticonspath}}/$emoticonspath/gi;
# $entryreturn =~ s/{{entrysuffix}}/$entrysuffix/gi;
# $entryreturn =~ s/{{archivesuffix}}/$logarchivesuffix/gi;
# $entryreturn =~ s/{{domain}}/$domain/gi;
# # end additions
# 
# # added by flipped cracker 1/26/05
# # version 1.3.1
# $entryreturn =~ s/&(?!#?[xX]?(?:[0-9a-fA-F]+|\w{1,8});)/&amp;/g;
# # end additions
# }


# ---------------------------------
# get log list variables
# ---------------------------------
## Goes through a specific entrylist and setups some variables

# LIST VARS ...

# ARG listEntry => a hash of entrylist line item data, see Gm_Storage
# (opt) ARG errHandler => the error handler used to display errors
# RETURN: a hash reference that contains the variables
# DEPRECATES: gm_getloglistvariables
# sub getLogListVariables {
# 
# 	my $listEntry = $params{'listEntry'};
# 	my $errorHandler = $params{'errHandler'} || \&Gm_Utils::gmWarn;
# 
# # $loglistnumberpadded = sprintf ("%8d", $loglistnumber);
# # $loglistnumberpadded =~ tr/ /0/;
# 	my loglistnumberpadded = Gm_Utils::toEntryPadded( $listEntry->{'id'} );
# 
# #($loglistmonthmonth, $loglistdayday, $loglistyear) = split (/\//, $loglistdate);
# 	my ($loglistmonthmonth, $loglistdayday, $loglistyear) = split (/\//, $listEntry->{'created'});
# 	
# # if ((substr($loglistmonthmonth, 0, 1)) eq "0") {
# # 	$loglistmonth = substr($loglistmonthmonth, -1, 1);
# # } else {
# # 	$loglistmonth = $loglistmonthmonth;
# # }
# 	my $loglistmonth = $loglistmonthmonth;
# 	if( (substr($loglistmonthmonth, 0, 1)) eq '0' ){
# 		$loglistmonth = substr($loglistmonthmonth, -1, 1);
# 	}
# 
# # if ((substr($loglistdayday, 0, 1)) eq "0") {
# # 	$loglistday = substr($loglistdayday, -1, 1);
# # } else {
# # 	$loglistday = $loglistdayday;
# # }
# 	my $loglistday = $loglistdayday;
# 	if ((substr($loglistdayday, 0, 1)) eq '0') {
# 		$loglistday = substr($loglistdayday, -1, 1);
# 	}
# 
# 	## THIS WILL CAUSE AN ISSUE IN 2081, BUT LETS FIX BY DOING 4 DIGIT YEAR ... TODO
# 	if ($loglistyear > 80) {
# 		$loglistyearyear = "19$loglistyear";
# 	} else {
# 		$loglistyearyear = "20$loglistyear";
# 	}
# 
# # ($loglisttime, $loglistampm) = split (/ /, $loglisttimeampm);
# # ($loglisthourhour, $loglistminuteminute) = split (/:/, $loglisttime);
# 	my ($loglisttime, $loglistampm) = split (' ', $listEntry->{'createt'});
# 	my ($loglisthourhour, $loglistminuteminute) = split (':', $loglisttime);
# 
# # if ((substr($loglisthourhour, 0, 1)) eq "0") {
# # 	$loglisthour = substr($loglisthourhour, -1, 1);
# # } else {
# # 	$loglisthour = $loglisthourhour;
# # }
# 	my $loglisthour = $loglisthourhour;
# 	if( (substr($loglisthourhour, 0, 1)) eq "0" ){
# 		$loglisthour = substr($loglisthourhour, -1, 1);
# 	}
# 
# # if ((substr($loglistminuteminute, 0, 1)) eq "0") {
# # 	$loglistminute = substr($loglistminuteminute, -1, 1);
# # } else {
# # 	$loglistminute = $loglistminuteminute;
# # }
# 	my $loglistminute = $loglistminuteminute;
# 	if ((substr($loglistminuteminute, 0, 1)) eq "0") {
# 		$loglistminute = substr($loglistminuteminute, -1, 1);
# 	}
# 
# 	## TODO: MORE MILITARY TIME STUFF, REFACTOR
# 	my $loglistmilitaryhour = $loglisthour;
# 
# # if ($loglistampm eq "AM") {
# # 	$loglistampmdot = "A.M.";
# # 	$loglistampmlower = "am";
# # 	$loglistampmdotlower = "a.m.";
# # } else {
# # 	$loglistampmdot = "P.M.";
# # 	$loglistampmlower = "pm";
# # 	$loglistampmdotlower = "p.m.";
# # 	$loglistmilitaryhour = $loglistmilitaryhour + 12;
# # }
# 	my $loglistampmdot = 'P.M.';
# 	my $loglistampmlower = 'pm';
# 	my $loglistampmdotlower = 'p.m.';
# 	if ($loglistampm eq 'AM') {
# 		$loglistampmdot = 'A.M.';
# 		$loglistampmlower = 'am';
# 		$loglistampmdotlower = 'a.m.';
# 	} else {
# 		$loglistmilitaryhour = $loglistmilitaryhour + 12;	
# 	}
# 
# # $loglistmilitaryhour = sprintf ("%2d", $loglistmilitaryhour);
# # $loglistmilitaryhour =~ tr/ /0/;
# 	$loglistmilitaryhour = Gm_Utils::toTwoDigit( $loglistmilitaryhour );
# 
# #@months = ("null", "Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio", "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre");
# 
# # $loglistmonthword = $months[$loglistmonth];
# # $loglistmonthwordshort = substr($loglistmonthword, 0, 3);
# # $loglistmonthwordupper = uc($loglistmonthword);
# # $loglistmonthwordlower = lc($loglistmonthword);
# # $loglistmonthworduppershort = uc($loglistmonthwordshort);
# # $loglistmonthwordlowershort = lc($loglistmonthwordshort);
# 	my $loglistmonthword = (Gm_Constants::MONTHS)[$loglistmonth-1];
# 	my $loglistmonthwordshort = (Gm_Constants::MONTHS_SHORT)[$loglistmonth-1];
# 	my $loglistmonthwordupper = uc($loglistmonthword);
# 	my $loglistmonthwordlower = lc($loglistmonthword);
# 	my $loglistmonthworduppershort = uc($loglistmonthwordshort);
# 	my $loglistmonthwordlowershort = lc($loglistmonthwordshort);
# 
# # open (FUNNYFEETLOGME, "$EntriesPath/$loglistnumberpadded.cgi") || &gm_dangermouse("Can't read $EntriesPath/$loglistnumberpadded.cgi.  Please make sure that your entries/archives directory is correctly configured and is CHMODed to 777, or try running Diagnostics & Repair in the Configuration screen.");
# # @loglistentrylines = <FUNNYFEETLOGME>;
# # close (FUNNYFEETLOGME);
# 	my $gmEntry = Gm_Storage::getEntry( id=>$listEntry->{'id'}, errHandler=>$errorHandler );
# 
# # $gmcounter = 0;  ## this variable isn't used in this sub like in other ones ...
# # 
# # foreach (@loglistentrylines) {
# # 	chomp ($loglistentrylines[$gmcounter]);
# # 	$gmcounter++;
# # }
# 
# # ($loglistentrynumber, $loglistentryauthor, $loglistentrysubject, 
# # $loglistentryweekdaynumber, $loglistentrymonth, $loglistentryday, 
# # $loglistentryyearyear, $loglistentryhour, $loglistentryminute, $loglistentrysecond, 
# # $loglistentryampm, $loglistentrypositivekarma, $loglistentrynegativekarma, 
# # $loglistentrycommentsnumber, $loglistentryallowkarma, $loglistentryallowcomments, 
# # $loglistentryopenstatus, $loglistentrymusic, $loglistentrymood, 
# # $loglistentryemoticonsallowed) = split (/\|/, $loglistentrylines[0]);
# 	my $loglistentrynumber = $gmEntry->{'entryinfo'}{'id'}; 
# 	my $loglistentryauthor = $gmEntry->{'entryinfo'}{'author'};
# 	my $loglistentrysubject = $gmEntry->{'entryinfo'}{'subject'};
# 	my $loglistentryweekdaynumber = $gmEntry->{'entryinfo'}{'weekday'}; 
# 	my $loglistentrymonth = $gmEntry->{'entryinfo'}{'month'};
# 	my $loglistentryday = $gmEntry->{'entryinfo'}{'day'};
# 	my $loglistentryyearyear = $gmEntry->{'entryinfo'}{'year'};
# 	my $loglistentryhour = $gmEntry->{'entryinfo'}{'hour'};
# 	my $loglistentryminute = $gmEntry->{'entryinfo'}{'minute'};
# 	my $loglistentrysecond = $gmEntry->{'entryinfo'}{'second'};
# 	my $loglistentryampm = $gmEntry->{'entryinfo'}{'ampm'};
# 	my $loglistentrypositivekarma = $gmEntry->{'entryinfo'}{'karmapos'}; 
# 	my $loglistentrynegativekarma = $gmEntry->{'entryinfo'}{'karmaneg'};
# 	my $loglistentrycommentsnumber = $gmEntry->{'entryinfo'}{'commenttotal'}; 
# 	my $loglistentryallowkarma = $gmEntry->{'entryinfo'}{'karma'};
# 	my $loglistentryallowcomments = $gmEntry->{'entryinfo'}{'comments'}; 
# 	my $loglistentryopenstatus = $gmEntry->{'entryinfo'}{'status'};
# 	my $loglistentrymusic = $gmEntry->{'entryinfo'}{'music'};
# 	my $loglistentrymood = $gmEntry->{'entryinfo'}{'mood'};
# 	my $loglistentryemoticonsallowed = $gmEntry->{'entryinfo'}{'emoticons'};
# 	
# 
# 	my $loglistentrytotalkarma = $loglistentrypositivekarma - $loglistentrynegativekarma;
# 
# # $loglistentrypositivekarmalink = "$cgiwebpath/gm-karma.cgi?vote=positive&entry=$loglistnumberpadded";
# # $loglistentrynegativekarmalink = "$cgiwebpath/gm-karma.cgi?vote=negative&entry=$loglistnumberpadded";
# 	my $loglistentrypositivekarmalink = $CONFIGS->{'gmcgiwebpath'}.
# 		"/gm-karma.cgi?vote=positive&entry=$loglistnumberpadded";
# 	my $loglistentrynegativekarmalink = $CONFIGS->{'gmcgiwebpath'}.
# 		"/gm-karma.cgi?vote=negative&entry=$loglistnumberpadded";
# 
# 	my $gmauthors = Gm_Storage::getAuthors( errHandler=>$errorHandler );
# 	my $selectedauthor;
# 	
# 	my $loglistentryauthoremail = Gm_Constants::EMPTY;
# 	my $loglistentryauthorhomepage = Gm_Constants::EMPTY;
# 	my $loglistentryauthorentrycount = Gm_Constants::EMPTY;
# 
# 	if( exists( $gmauthors->{$thisentryauthor} )){
# 		$selectedauthor = $gmauthors->{$thisentryauthor};
# 		$loglistentryauthoremail = $selectedauthor->{'email'};
# 		$loglistentryauthorhomepage = $selectedauthor->{'homepage'};
# 		$loglistentryauthorentrycount = $selectedauthor->{'posttotal'};
# 	}
# 
# 	my $loglistentryfilename = "$EntriesWebPath\/$loglistnumberpadded\.$entrysuffix";
# 	my $loglistentrycommentspostlink = "$loglistentryfilename\#comments";
# 
# 	my $loglistentrycommentstatussmart = $gmsmartlinkmanycommentstemplate;
# # if ($loglistentrycommentsnumber eq "0") { $loglistentrycommentstatussmart = $gmsmartlinknocommentstemplate; }
# # if ($loglistentrycommentsnumber eq "1") { $loglistentrycommentstatussmart = $gmsmartlinkonecommenttemplate; }
# 	if( $loglistentrycommentsnumber < 1 ){ 
# 		$loglistentrycommentstatussmart = $gmsmartlinknocommentstemplate; 
# 	}
# 	if( $loglistentrycommentsnumber == 1 ){ 
# 		$loglistentrycommentstatussmart = $gmsmartlinkonecommenttemplate; 
# 	}
# 
# 	my $loglistentrycommentstatussmartupper = uc( $loglistentrycommentstatussmart );
# 	my $loglistentrycommentstatussmartlower = lc( $loglistentrycommentstatussmart );
# 
# # 	my $leapyearcheck = $loglistyearyear % 4;
# # 	## THIS IS DUPLICATED CODE, TODO: REFACTOR
# # 	my $loglistmaxdaysinthismonth = 31;
# # 	if( ($loglistmonthword eq 'September') || ($loglistmonthword eq 'April') || 
# # 		($loglistmonthword eq 'June') || ($loglistmonthword eq 'November') ){ 
# # 		$loglistmaxdaysinthismonth = 30; 
# # 	}
# # 	if( $loglistmonthword eq 'February' ) {
# # 		$loglistmaxdaysinthismonth = 28;
# # 		if( $leapyearcheck eq '0' ) { 
# # 			$loglistmaxdaysinthismonth = 29; 
# # 		}
# # 	}
# # 	## TODO: REFACTOR THIS TO A GENERIC FUNCTION THAT DOESN'T DO AN EXACT STRING MATCH
# # 	my $loglistmaxdaysinpreviousmonth = 31;
# # 	if( ($loglistmonthword eq 'October') || ($loglistmonthword eq 'May') || 
# # 		($loglistmonthword eq 'July') || ($loglistmonthword eq 'December') ){ 
# # 			$loglistmaxdaysinpreviousmonth = 30; 
# # 	}
# # 	if( $loglistmonthword eq 'March' ){
# # 		$loglistmaxdaysinpreviousmonth = 28;
# # 		if( $leapyearcheck eq '0' ){ 
# # 			$loglistmaxdaysinpreviousmonth = 29; 
# # 		}
# # 	}
# 	my $loglistmaxdaysinthismonth = Gm_Utils::getDays( $loglistmonth, $loglistyearyear  );
# 	my $loglistmaxdaysinpreviousmonth = '';
# 	if( $loglistmonth == 1 ){
# 		$loglistmaxdaysinpreviousmonth = Gm_Utils::getDays( 12, $loglistyearyear-1  );
# 	} else {
# 		$loglistmaxdaysinpreviousmonth = Gm_Utils::getDays( $loglistmonth-1, $loglistyearyear  );
# 	}
# 
# 	my $loglistweekbeginningmonth = $loglistmonth;
# 	my $loglistweekbeginningyearyear = $loglistyearyear;
# 	my $loglistweekendingmonth = $loglistmonth;
# 	my $loglistweekendingyearyear = $loglistyearyear;
# 
# 	my $loglistweekbeginningday = $loglistday - $loglistentryweekdaynumber;
# 	my $loglistweekendingday = $loglistweekbeginningday + 6;
# 
# 	if ($loglistweekbeginningday < 1) {
# 		$loglistweekbeginningday = $loglistweekbeginningday + $loglistmaxdaysinpreviousmonth;
# 		if( $loglistweekbeginningday > $loglistday ){ 
# 			$loglistweekbeginningmonth--; 
# 		}
# 		if ($loglistweekbeginningmonth < 1) {
# 			$loglistweekbeginningmonth = 12;
# 			$loglistweekbeginningyearyear--;
# 		}
# 	}
# 
# 	if ($loglistweekendingday > $loglistmaxdaysinthismonth) {
# 		$loglistweekendingday = $loglistweekendingday - $loglistmaxdaysinthismonth;
# 		if ($loglistweekendingday < $loglistday) { $loglistweekendingmonth++; }
# 		if ($loglistweekendingmonth > 12) {
# 			$loglistweekendingmonth = 1;
# 			$loglistweekendingyearyear++;
# 		}
# 	}
# 
# 	my $loglistweekbeginningyear = substr($loglistweekbeginningyearyear, -2, 2);
# 	my $loglistweekendingyear = substr($loglistweekendingyearyear, -2, 2);
# 
# # $loglistweekbeginningdayday = sprintf ("%2d", $loglistweekbeginningday);
# # $loglistweekbeginningdayday =~ tr/ /0/;
# # $loglistweekendingdayday = sprintf ("%2d", $loglistweekendingday);
# # $loglistweekendingdayday =~ tr/ /0/;
# # $loglistweekbeginningmonthmonth = sprintf ("%2d", $loglistweekbeginningmonth);
# # $loglistweekbeginningmonthmonth =~ tr/ /0/;
# # $loglistweekendingmonthmonth = sprintf ("%2d", $loglistweekendingmonth);
# # $loglistweekendingmonthmonth =~ tr/ /0/;
# 	my $loglistweekbeginningdayday = Gm_Utils::toTwoDigit( $loglistweekbeginningday);
# 	my $loglistweekendingdayday = Gm_Utils::toTwoDigit( $loglistweekendingday);
# 	my $loglistweekbeginningmonthmonth = Gm_Utils::toTwoDigit( $loglistweekbeginningmonth);
# 	my $loglistweekendingmonthmonth = Gm_Utils::toTwoDigit( $loglistweekendingmonth);
# 
# 	## TODO: ALLOW USER TO DETERMINE WHICH DAY WEEK START ON, IF DO, NEED TO UPDATE HERE!
# # $loglistweekbeginningweekday = 'Sunday';
# 	my $loglistweekbeginningweekday = = (Gm_Constants::DAYS)[0];
# # $loglistweekbeginningmonthword = $months[$loglistweekbeginningmonth];
# 	my $loglistweekbeginningmonthword = (Gm_Constants::MONTHS)[$loglistweekbeginningmonth-1];
# # $loglistweekbeginningweekdayshort = substr($loglistweekbeginningweekday, 0, 3);
# 	my $loglistweekbeginningweekdayshort = (Gm_Constants::DAYS_SHORT)[0];
# # $loglistweekbeginningmonthwordshort = substr($loglistweekbeginningmonthword, 0, 3);
# 	my $loglistweekbeginningmonthwordshort = (Gm_Constants::MONTHS_SHORT)[$loglistweekbeginningmonth-1];
# 
# 	## TODO: MAKE A GENERIC FUNCTION THAT RETURNS A HAS OF LC AND UC 
# 	my $loglistweekbeginningweekdayupper = uc($loglistweekbeginningweekday);
# 	my $loglistweekbeginningmonthwordupper = uc($loglistweekbeginningmonthword);
# 	my $loglistweekbeginningweekdaylower = lc($loglistweekbeginningweekday);
# 	my $loglistweekbeginningmonthwordlower = lc($loglistweekbeginningmonthword);
# 	my $loglistweekbeginningweekdayuppershort = uc($loglistweekbeginningweekdayshort);
# 	my $loglistweekbeginningmonthworduppershort = uc($loglistweekbeginningmonthwordshort);
# 	my $loglistweekbeginningweekdaylowershort = lc($loglistweekbeginningweekdayshort);
# 	my $loglistweekbeginningmonthwordlowershort = lc($loglistweekbeginningmonthwordshort);
# 
# ## TODO: ALLOW USER TO DETERMINE WHICH DAY WEEK START ON, IF DO, NEED TO UPDATE HERE!
# # $loglistweekendingweekday = 'Saturday';
# 	my $loglistweekendingweekday = (Gm_Constants::DAYS)[6];
# # $loglistweekendingmonthword = $months[$loglistweekendingmonth];
# 	my $loglistweekendingmonthword = (Gm_Constants::MONTHS)[$loglistweekendingmonth-1];
# # $loglistweekendingweekdayshort = substr($loglistweekendingweekday, 0, 3);
# 	my $loglistweekendingweekdayshort = (Gm_Constants::DAYS_SHORT)[6];
# # $loglistweekendingmonthwordshort = substr($loglistweekendingmonthword, 0, 3);
# 	my $loglistweekendingmonthwordshort = (Gm_Constants::MONTHS_SHORT)[$loglistweekendingmonth-1];
# 
# 	## TODO: MAKE A GENERIC FUNCTION THAT RETURNS A HAS OF LC AND UC
# 	my $loglistweekendingweekdayupper = uc($loglistweekendingweekday);
# 	my $loglistweekendingmonthwordupper = uc($loglistweekendingmonthword);
# 	my $loglistweekendingweekdaylower = lc($loglistweekendingweekday);
# 	my $loglistweekendingmonthwordlower = lc($loglistweekendingmonthword);
# 	my $loglistweekendingweekdayuppershort = uc($loglistweekendingweekdayshort);
# 	my $loglistweekendingmonthworduppershort = uc($loglistweekendingmonthwordshort);
# 	my $loglistweekendingweekdaylowershort = lc($loglistweekendingweekdayshort);
# 	my $loglistweekendingmonthwordlowershort = lc($loglistweekendingmonthwordshort);
# 
# # $loglistpagelink = "$EntriesWebPath\/$loglistnumberpadded\.$entrysuffix";
# 	my $loglistpagelink = $CONFIGS->{'gmentrieswebpath'}.'/'.
# 		$loglistnumberpadded.'.'.$CONFIGS->{'gmentrysuffix'};
# 
# 	## TODO: THIS IS FOCUSING HEAVILY ON THE ASSUMPTION THAT THE FILES ARE GOING TO BE
#   # PREGENERATED, WHICH IS FINE, BUT IF THIS IS REFACTORED TO BE DYNAMIC, NOTE THIS AREA! 
#   my $loglistpagearchivelogindexlink = '';
# 	if( $CONFIGS->{'gmkeepmonthlyarchives'} eq Gm_Constants::NO ){
# 		$loglistpagearchivelogindexlink = '{{pageindexlink}}';
# 	} else {
# 		if( $CONFIGS->{'gmarchiveformat'} eq Gm_Constants::WEEK ){
# #			$loglistpagearchivelogindexlink = "$EntriesWebPath/archivo-$loglistweekbeginningmonthmonth$loglistweekbeginningdayday
# #$loglistweekbeginningyearyear-$loglistweekendingmonthmonth$loglistweekendingdayday$loglistweekendingyearyear\.$logarchivesuffix";
# 			$loglistpagearchivelogindexlink = $CONFIGS->{'gmentrieswebpath'}.
# 				"/archivo-$loglistweekbeginningmonthmonth$loglistweekbeginningdayday".
# 				"$loglistweekbeginningyearyear-$loglistweekendingmonthmonth".
# 				"$loglistweekendingdayday$loglistweekendingyearyear\.".
# 				$CONFIGS->{'gmlogarchivesuffix'};
# 		} else {
# #			$loglistpagearchivelogindexlink = "$EntriesWebPath/archivo-$loglistmonthmonth$loglistyearyear\.
# #$logarchivesuffix";
# 			$thisentrypagearchivelogindexlink = $CONFIGS->{'gmentrieswebpath'}.
#       	"/archivo-$loglistmonthmonth$loglistyearyear\.".$CONFIGS->{'gmlogarchivesuffix'};
# 		}
# 	}
# 
# # $loglistmainbody = $loglistentrylines[2];
# # $loglistmorebody = $loglistentrylines[3];
# 	my $loglistmainbody = $gmEntry->{'maintext'};
# 	my $loglistmorebody = $gmEntry->{'extendedtext'};
# 
# 
# 
# 
# 
# if (($loglistmainbody =~ /\|\*\|/) || ($loglistmorebody =~ /\|\*\|/)) {
# 	$loglistmainbody =~ s/\|\*\|/<BR>/g;
# 	$loglistmorebody =~ s/\|\*\|/<BR>/g;
# }
# if (($loglistmainbody =~ /\n/) || ($loglistmorebody =~ /\n/)) {
# 	$loglistmainbody =~ s/\n/<BR>/g;
# 	$loglistmorebody =~ s/\n/<BR>/g;
# }
# if (($loglistmainbody =~ /<BR><BR>/) || ($loglistmorebody =~ /<BR><BR>/)) {
# 	$loglistmainbody =~ s/<BR><BR>/$gmparaseparationtemplate/g;
# 	$loglistmorebody =~ s/<BR><BR>/$gmparaseparationtemplate/g;
# }
# 
# if (($censorenabled eq Gm_Constants::BOTH) || ($censorenabled eq Gm_Constants::ENTRIES)) {
# 	unless ($censorlist eq Gm_Constants::EMPTY) {
# 		@censoredterms = split(/\n/, $censorlist);
# 		foreach $thisterm (@censoredterms) {
# 			unless ($thisterm eq Gm_Constants::EMPTY) {
# 				if ((substr($thisterm, 0, 1) eq '[') && (substr($thisterm, -1, 1) eq ']')) {
# 					$thisrealterm = $thisterm;
# 					$thisrealterm =~ s/\[//g;
# 					$thisrealterm =~ s/\]//g;
# 					$thisrealtermlength = length($thisrealterm);
# 					$thisrealtermreplace = "*" x $thisrealtermlength;
# 					$thisrealtermreplacedash = "-" x $thisrealtermlength;
# 					if ($loglistsubject =~ m/$thisrealterm/i) {
# 						$loglistsubject =~ s/\b$thisrealterm\b/$thisrealtermreplacedash/isg;
# 					}
# 					if ($loglistmainbody =~ m/$thisrealterm/i) {
# 						$loglistmainbody =~ s/\b$thisrealterm\b/$thisrealtermreplace/isg;
# 					}
# 					if ($loglistmorebody =~ m/$thisrealterm/i) {
# 						$loglistmorebody =~ s/\b$thisrealterm\b/$thisrealtermreplace/isg;
# 					}
# 					if ($loglistentryauthor =~ m/$thisrealterm/i) {
# 						$loglistentryauthor =~ s/\b$thisrealterm\b/$thisrealtermreplacedash/isg;
# 					}
# 				} else {
# 					$thisrealtermlength = length($thisterm);
# 					$thisrealtermreplace = "*" x $thisrealtermlength;
# 					$thisrealtermreplacedash = "-" x $thisrealtermlength;
# 					if ($loglistsubject =~ m/$thisterm/i) {
# 						$loglistsubject =~ s/\b$thisterm\b/$thisrealtermreplacedash/isg;
# 					}
# 					if ($loglistmainbody =~ m/$thisterm/i) {
# 						$loglistmainbody =~ s/\b$thisterm\b/$thisrealtermreplace/isg;
# 					}
# 					if ($loglistmorebody =~ m/$thisterm/i) {
# 						$loglistmorebody =~ s/\b$thisterm\b/$thisrealtermreplace/isg;
# 					}
# 					if ($loglistentryauthor =~ m/$thisterm/i) {
# 						$loglistentryauthor =~ s/\b$thisterm\b/$thisrealtermreplacedash/isg;
# 					}
# 				}
# 			}
# 		}
# 	}
# }
# 
# ### NOTE: this must be done after censoring or smartlink won't pick up censoring
# ## TODO: MAKE A GETSMARTAUTHORLINKS FOR HOMEPAGE AND EMAIL SINCE THIS IS DUPPED +3 TIMES
# # $loglistentryauthorsmartlink = $loglistentryauthor;
# # if ($loglistentryauthoremail ne Gm_Constants::EMPTY) { $loglistentryauthorsmartlink = "<A HREF=\"mailto:$loglistentryauthoremail\">$loglistentryauthor</A>"; }
# # if ($loglistentryauthorhomepage ne Gm_Constants::EMPTY) { $loglistentryauthorsmartlink = "<A HREF=\"$loglistentryauthorhomepage\">$loglistentryauthor</A>"; }
# 	my $loglistentryauthorsmartlink = $loglistentryauthor;
# 	if( $loglistentryauthoremail ne Gm_Constants::EMPTY ){ 
# 		$loglistentryauthorsmartlink = "<A HREF=\"mailto:$loglistentryauthoremail\">$loglistentryauthor</A>"; 
# 	}
# 	if( $loglistentryauthorhomepage ne Gm_Constants::EMPTY ){ 
# 		$loglistentryauthorsmartlink = "<A HREF=\"$loglistentryauthorhomepage\">$loglistentryauthor</A>"; 
# 	}
# 
# # changed deprecated <b>, <i>, and <u> to corresponding css properties: flipped cracker 9/15/2003
# # removed code for entry subject : 1.3.1
# # flipped cracker 6/29/2004
# if (($inlineformatting eq Gm_Constants::ENTRIES) || ($inlineformatting eq Gm_Constants::BOTH)) {
# 	if (($loglistmainbody =~ /\*\*(.*?)\*\*/) || ($loglistmainbody =~ /\\\\(.*?)\\\\/) || ($loglistmainbody =~ /__(.*?)__/)) {
# 		$loglistmainbody =~ s/\*\*(.*?)\*\*/<span style="font-weight:bold;">$1<\/span>/isg;
# 		$loglistmainbody =~ s/\\\\(.*?)\\\\/<span style="font-style:italic;">$1<\/span>/isg;
# 		$loglistmainbody =~ s/__(.*?)__/<span style="text-decoration:underline;">$1<\/span>/isg;
# 	}
# 	if (($loglistmorebody =~ /\*\*(.*?)\*\*/) || ($loglistmorebody =~ /\\\\(.*?)\\\\/) || ($loglistmorebody =~ /__(.*?)__/)) {
# 		$loglistmorebody =~ s/\*\*(.*?)\*\*/<span style="font-weight:bold;">$1<\/span>/isg;
# 		$loglistmorebody =~ s/\\\\(.*?)\\\\/<span style="font-style:italic;">$1<\/span>/isg;
# 		$loglistmorebody =~ s/__(.*?)__/<span style="text-decoration:underline;">$1<\/span>/isg;
# 	}
# }
# #changed 9/15/2003
# 
# }



# ---------------------------------
# get entry variables
# ---------------------------------
## Sets up many variables based on the given entry that can be used for
# token substitution:

# 	thisentrynumber => the entry number, without leading zeros
# 	thisentryauthor => the author of the entry
# 	thisentrysubject => subject of entry
# 	thisentryweekdaynumber => a number between 0 and 9 (Sun - Sat)
# 	thisentrymonth => month number, no leading zeros
# 	thisentryday => day number, no leading zeros
# 	thisentryyearyear	=> year, four digits
# 	thisentryhour	=> hour of entry, no leading zero
# 	thisentryminute	=> minutes of entry, no leading zero
# 	thisentrysecond	=> seconds of entry, no leading zero
# 	thisentryampm	=> whether its AM or PM, all caps, no periods
# 	thisentrypositivekarma => number of positive karma
# 	thisentrynegativekarma => number of negative karma
# 	thisentrycommentsnumber	=> number of comments
# 	thisentryallowkarma	=> whether this entry allows karma voting
# 	thisentryallowcomments => whether this entry allows comments
# 	thisentryopenstatus	=> whether this entry is open
# 	thisentrymusic => the music of the entry
# 	thisentrymood	=> the mood of the entry
# 	thisentryemoticonsallowed => whether emoticons are allowed
# 	thisentryyear => the two digit year
# 	thisentryweekday => the name of the week day Sunday - Saturday
# 	thisentryweekdayshort => the name of the week day short Sun - Sat
# 	thisentrymonthword => the name of the month January - December
# 	thisentrymonthwordshort => the name of the month short Jan - Dec
# 	thisentryweekdayupper
# 	thisentrymonthwordupper
# 	thisentryweekdaylower
# 	thisentrymonthwordlower
# 	thisentryweekdayuppershort
# 	thisentrymonthworduppershort
# 	thisentryweekdaylowershort
# 	thisentrymonthwordlowershort
# 	thisentrymonthmonth
# 	thisentrydayday
# 	thisentryhourhour
# 	thisentryminuteminute
# 	thisentrysecondsecond
# 	thisentryampmdot
# 	thisentrymilitaryhour
# 	thisentryampmlower
# 	thisentryampmdotlower
# 	thisentryweekbeginningmonth
# 	thisentryweekbeginningyearyear
# 	thisentryweekendingmonth
# 	thisentryweekendingyearyear
# 	thisentryweekbeginningday
# 	thisentryweekendingday
# 	thisentryweekbeginningyear
# 	thisentryweekendingyear
# 	thisentryweekbeginningdayday
# 	thisentryweekendingdayday
# 	thisentryweekbeginningmonthmonth
# 	thisentryweekendingmonthmonth
# 	thisentryweekbeginningweekday
# 	thisentryweekbeginningmonthword
# 	thisentryweekbeginningweekdayshort
# 	thisentryweekbeginningmonthwordshort
# 	thisentryweekbeginningweekdayupper
# 	thisentryweekbeginningmonthwordupper
# 	thisentryweekbeginningweekdaylower
# 	thisentryweekbeginningmonthwordlower
# 	thisentryweekbeginningweekdayuppershort
# 	thisentryweekbeginningmonthworduppershort
# 	thisentryweekbeginningweekdaylowershort
# 	thisentryweekbeginningmonthwordlowershort
# 	thisentryweekendingweekday
# 	thisentryweekendingmonthword
# 	thisentryweekendingweekdayshort
# 	thisentryweekendingmonthwordshort
# 	thisentryweekendingweekdayupper
# 	thisentryweekendingmonthwordupper
# 	thisentryweekendingweekdaylower
# 	thisentryweekendingmonthwordlower
# 	thisentryweekendingweekdayuppershort
# 	thisentryweekendingmonthworduppershort
# 	thisentryweekendingweekdaylowershort
# 	thisentryweekendingmonthwordlowershort
# 	thisentryauthoremail
# 	thisentryauthorhomepage
# 	thisentryauthorentrycount
# 	thisentrycommentspostlink
# 	thisentrycommentstatussmart
# 	thisentrycommentstatussmartupper
# 	thisentrycommentstatussmartlower
# 	thisentrypagelink
# 	thisentrypageindexlink
# 	thisentrypagearchiveindexlink
# 	thisentrypagearchivelogindexlink
# 	thisentrypagesmartindexlink
# 	thisentrycommentslink
# 	thisentrycommentsform	
# 	thisentrykarmalink
# 	thisentrykarmaform
# 	thisentrysearchform
# 	thisentrypositivekarmalink
# 	thisentrynegativekarmalink
# 	thisentrytotalkarma
# 	thisentrymainbody
# 	thisentrymorebody
# 	thisentryauthorsmartlink
# 	thisentrycommentdivider
# 	thisentrycomments

# ARG entryid => the id of the entry to work on
# (opt) ARG errHandler => the error handler used to display errors
# RETURN: a hash reference that contains the variables
# DEPRECATES: gm_getentryvariables
sub getEntryVariables {
	my (%params) = @_;

	## Setting up vars
	my %entryVars = ();
	my $thisentrygetmynumber = $params{'entryid'};
	my $errorHandler = $params{'errHandler'} || \&Gm_Utils::gmWarn;

	## Gotta have an entry otherwise, whats point?  Since no vars setup, lets stop here
	unless( $thisentrygetmynumber ){
		&$errorHandler( 'An unrecoverable error occured: Invalid id passed to getEntryVariables.' );
	}

# &gm_readconfig;
# &gm_readtemplates;
# &gm_readcounter;

	
	my $gmCounters = Gm_Storage::getCounters( errHandler=>$errorHandler );

# $thisentrynumberpadded = sprintf ("%8d", $thisentrygetmynumber);
# $thisentrynumberpadded =~ tr/ /0/;
	my $thisentrynumberpadded = Gm_Utils::toEntryPadded( $thisentrygetmynumber );

	my $gmEntry = Gm_Storage::getEntry( id=>$thisentrygetmynumber,
		errHandler=>$errorHandler );

# open (FUNNYFEETENTRY, "$EntriesPath/$thisentrynumberpadded.cgi") || &gm_dangermouse("Can't read $EntriesPath/$thisentrynumberpadded.cgi.  Please make sure that your entries/archives directory is correctly configured and is CHMODed to 777, or try running Diagnostics & Repair in the Configuration screen.");
# @entrylines = <FUNNYFEETENTRY>;
# close (FUNNYFEETENTRY);

# $gmcounter = 0;
# 
# foreach (@entrylines) {
# 	chomp ($entrylines[$gmcounter]);
# 	$entrylines[$gmcounter] =~ s/\|\*\|/\n/g;
# 	$gmcounter++;
# }

# ($thisentrynumber, $thisentryauthor, $thisentrysubject, $thisentryweekdaynumber, 
# $thisentrymonth, $thisentryday, $thisentryyearyear, $thisentryhour, 
# $thisentryminute, $thisentrysecond, $thisentryampm, $thisentrypositivekarma, 
# $thisentrynegativekarma, $thisentrycommentsnumber, $thisentryallowkarma, 
# $thisentryallowcomments, $thisentryopenstatus, $thisentrymusic, 
# $thisentrymood, $thisentryemoticonsallowed) = split (/\|/, $entrylines[0]);

## TODO: refactor out usage of variables, make work directly off of entry
	my $thisentrynumber = $gmEntry->{'entryinfo'}{'id'};
	my $thisentryauthor = $gmEntry->{'entryinfo'}{'author'};
	my $thisentrysubject = $gmEntry->{'entryinfo'}{'subject'};
	my $thisentryweekdaynumber = $gmEntry->{'entryinfo'}{'weekday'};
	my $thisentrymonth = $gmEntry->{'entryinfo'}{'month'};
	my $thisentryday = $gmEntry->{'entryinfo'}{'day'};
	my $thisentryyearyear = $gmEntry->{'entryinfo'}{'year'};
	my $thisentryhour = $gmEntry->{'entryinfo'}{'hour'};
	my $thisentryminute = $gmEntry->{'entryinfo'}{'minute'};
	my $thisentrysecond = $gmEntry->{'entryinfo'}{'second'};
	my $thisentryampm = $gmEntry->{'entryinfo'}{'ampm'};
	my $thisentrypositivekarma = $gmEntry->{'entryinfo'}{'karmapos'};
	my $thisentrynegativekarma = $gmEntry->{'entryinfo'}{'karmaneg'};
	my $thisentrycommentsnumber = $gmEntry->{'entryinfo'}{'commenttotal'};
	my $thisentryallowkarma = $gmEntry->{'entryinfo'}{'karma'};
	my $thisentryallowcomments = $gmEntry->{'entryinfo'}{'comments'};
	my $thisentryopenstatus = $gmEntry->{'entryinfo'}{'status'};
	my $thisentrymusic = $gmEntry->{'entryinfo'}{'music'};
	my $thisentrymood = $gmEntry->{'entryinfo'}{'mood'};
	my $thisentryemoticonsallowed = $gmEntry->{'entryinfo'}{'emoticons'};
	
# chomp ($thisentryemoticonsallowed);

# my $thisentryisanarchive = Gm_Constants::NO;
# if( $thisentrynumber <= $newarchivenumber ){ 
# 	$thisentryisanarchive = Gm_Constants::YES; 
# }
	my $thisentryisanarchive = Gm_Constants::NO; 
	## checking the counter to see if this is an archived entry
	if( $thisentrynumber <= $gmCounters->{'archivetotal'} ){ 
		$thisentryisanarchive = Gm_Constants::YES; 
	}

# if ($generateentrypages eq Gm_Constants::NO) { $thisentryallowcomments = Gm_Constants::NO; }
	if( $CONFIGS->{'gmgenerateentrypages'} eq Gm_Constants::NO ){ 
		$thisentryallowcomments = Gm_Constants::NO; 
	}

## THIS IS SIMILAR TO GM_UTILS::GETSTDDATE, HOWEVER, ITS WORKING OFF OF A PREDETERMINED DATE
	my $thisentryyear = substr($thisentryyearyear, -2, 2);

# @months = ("null", "Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio", "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre");
# @weekdays = ("Domingo", "Lunes", "Martes", "Miercoles", "Jueves", "Viernes", "Sabado");

	my $thisentryweekday = (Gm_Constants::DAYS)[$thisentryweekdaynumber];
	my $thisentryweekdayshort = (Gm_Constants::DAYS_SHORT)[$thisentryweekdaynumber];
	my $thisentrymonthword = (Gm_Constants::MONTHS)[$thisentrymonth-1];
	my $thisentrymonthwordshort = (Gm_Constants::MONTHS_SHORT)[$thisentrymonth-1];

	## This is for tag replacement, TODO: MAKE GENERIC SO RETURNS HAS WITH LC UC VALS?
	my $thisentryweekdayupper = uc($thisentryweekday);
	my $thisentrymonthwordupper = uc($thisentrymonthword);
	my $thisentryweekdaylower = lc($thisentryweekday);
	my $thisentrymonthwordlower = lc($thisentrymonthword);
	my $thisentryweekdayuppershort = uc($thisentryweekdayshort);
	my $thisentrymonthworduppershort = uc($thisentrymonthwordshort);
	my $thisentryweekdaylowershort = lc($thisentryweekdayshort);
	my $thisentrymonthwordlowershort = lc($thisentrymonthwordshort);

# $thisentrymonthmonth = sprintf ("%2d", $thisentrymonth);
# $thisentrydayday = sprintf ("%2d", $thisentryday);
# $thisentryhourhour = sprintf ("%2d", $thisentryhour);
# $thisentryminuteminute = sprintf ("%2d", $thisentryminute);
# $thisentrysecondsecond = sprintf ("%2d", $thisentrysecond);
# $thisentrymonthmonth =~ tr/ /0/;
# $thisentrydayday =~ tr/ /0/;
# $thisentryhourhour =~ tr/ /0/;
# $thisentryminuteminute =~ tr/ /0/;
# $thisentrysecondsecond =~ tr/ /0/;

	## This is for tag replacement, TODO: MAKE GENERIC SO RETURNS HAS WITH LC UC VALS?
	my $thisentrymonthmonth = Gm_Utils::toTwoDigit( $thisentrymonth );
	my $thisentrydayday = Gm_Utils::toTwoDigit( $thisentryday );
	my $thisentryhourhour = Gm_Utils::toTwoDigit( $thisentryhour );
	my $thisentryminuteminute = Gm_Utils::toTwoDigit( $thisentryminute );
	my $thisentrysecondsecond = Gm_Utils::toTwoDigit( $thisentrysecond );

	my $thisentryampmdot = 'A.M.';
	my $thisentrymilitaryhour = $thisentryhour;
	if( $thisentryampm eq 'PM' ){
		$thisentryampmdot = 'P.M.';
		if( $thisentryhour ne '12' ){ 
			$thisentrymilitaryhour = $thisentryhour + 12; 
		}
	}

# $thisentrymilitaryhour = sprintf ("%2d", $thisentrymilitaryhour);
# $thisentrymilitaryhour =~ tr/ /0/;
	$thisentrymilitaryhour = Gm_Utils::toTwoDigit( $thisentrymilitaryhour );
	my $thisentryampmlower = lc($thisentryampm);
	my $thisentryampmdotlower = lc($thisentryampmdot);

	# Military hour is 00 for 12 AM
	if (($thisentryhour eq '12') && ($thisentryampm eq 'AM')) {
		$thisentrymilitaryhour = '00';
	}

#$leapyearcheck = $thisentryyearyear % 4;

# $thisentrymaxdaysinthismonth = 31;
# if (($thisentrymonthword eq "September") || ($thisentrymonthword eq "April") || ($thisentrymonthword eq "June") || ($thisentrymonthword eq "November")) { $thisentrymaxdaysinthismonth = 30; }
# if ($thisentrymonthword eq "February") {
# 	$thisentrymaxdaysinthismonth = 28;
# 	if ($leapyearcheck eq "0") { $thisentrymaxdaysinthismonth = 29; }
# }
# $thisentrymaxdaysinpreviousmonth = 31;
# if (($thisentrymonthword eq "October") || ($thisentrymonthword eq "May") || ($thisentrymonthword eq "July") || ($thisentrymonthword eq "December")) { $thisentrymaxdaysinpreviousmonth = 30; }
# if ($thisentrymonthword eq "March") {
# 	$thisentrymaxdaysinpreviousmonth = 28;
# 	if ($leapyearcheck eq "0") { $thisentrymaxdaysinpreviousmonth = 29; }
# }

	my $thisentrymaxdaysinthismonth = Gm_Utils::getDays( $thisentrymonth, $thisentryyearyear  );
#	my $thisentrymaxdaysinpreviousmonth = Gm_Utils::getDays( $thisentrymonth, $thisentryyearyear );
	my $thisentrymaxdaysinpreviousmonth = '';
	if( $thisentrymonth == 1 ){
		$thisentrymaxdaysinpreviousmonth = Gm_Utils::getDays( 12, $thisentryyearyear-1  );
	} else {
		$thisentrymaxdaysinpreviousmonth = Gm_Utils::getDays( $thisentrymonth-1, $thisentryyearyear  );
	}

	## TODO: REFACTOR THIS DATE MAGIC, CAN CREATE SEVERAL UTIL FUNCTIONS HERE 
	my $thisentryweekbeginningmonth = $thisentrymonth;
	my $thisentryweekbeginningyearyear = $thisentryyearyear;
	my $thisentryweekendingmonth = $thisentrymonth;
	my $thisentryweekendingyearyear = $thisentryyearyear;

	my $thisentryweekbeginningday = $thisentryday - $thisentryweekdaynumber;
	my $thisentryweekendingday = $thisentryweekbeginningday + 6;
	
	## Seems to be getting next year, if the curr month is Dec ?, make func, get next month, year?
	if ($thisentryweekbeginningday < 1) {
		$thisentryweekbeginningday = $thisentryweekbeginningday + $thisentrymaxdaysinpreviousmonth;
		if( $thisentryweekbeginningday > $thisentryday ){ 
			$thisentryweekbeginningmonth--; 
		}
		if( $thisentryweekbeginningmonth < 1 ){
			$thisentryweekbeginningmonth = 12;
			$thisentryweekbeginningyearyear--;
		}
	}
	## Seems to be getting next year, if the curr month is Dec ?, make func, get next month, year?
	if ($thisentryweekendingday > $thisentrymaxdaysinthismonth) {
		$thisentryweekendingday = $thisentryweekendingday - $thisentrymaxdaysinthismonth;
		if ($thisentryweekendingday < $thisentryday) { $thisentryweekendingmonth++; }
		if ($thisentryweekendingmonth > 12) {
			$thisentryweekendingmonth = 1;
			$thisentryweekendingyearyear++;
		}
	}

	my $thisentryweekbeginningyear = substr($thisentryweekbeginningyearyear, -2, 2);
	my $thisentryweekendingyear = substr($thisentryweekendingyearyear, -2, 2);

# $thisentryweekbeginningdayday = sprintf ("%2d", $thisentryweekbeginningday);
# $thisentryweekbeginningdayday =~ tr/ /0/;
# $thisentryweekendingdayday = sprintf ("%2d", $thisentryweekendingday);
# $thisentryweekendingdayday =~ tr/ /0/;
# $thisentryweekbeginningmonthmonth = sprintf ("%2d", $thisentryweekbeginningmonth);
# $thisentryweekbeginningmonthmonth =~ tr/ /0/;
# $thisentryweekendingmonthmonth = sprintf ("%2d", $thisentryweekendingmonth);
# $thisentryweekendingmonthmonth =~ tr/ /0/;

	my $thisentryweekbeginningdayday = Gm_Utils::toTwoDigit( $thisentryweekbeginningday);
	my $thisentryweekendingdayday = Gm_Utils::toTwoDigit( $thisentryweekendingday);
	my $thisentryweekbeginningmonthmonth = Gm_Utils::toTwoDigit( $thisentryweekbeginningmonth);
	my $thisentryweekendingmonthmonth = Gm_Utils::toTwoDigit( $thisentryweekendingmonth);

	## TODO: ALLOW USER TO DETERMINE WHICH DAY WEEK START ON, IF DO, NEED TO UPDATE HERE!
# $thisentryweekbeginningweekday = 'Sunday';
	my $thisentryweekbeginningweekday = (Gm_Constants::DAYS)[0];
# $thisentryweekbeginningmonthword = $months[$thisentryweekbeginningmonth];
	my $thisentryweekbeginningmonthword = (Gm_Constants::MONTHS)[$thisentryweekbeginningmonth-1];
# $thisentryweekbeginningweekdayshort = substr($thisentryweekbeginningweekday, 0, 3);
	my $thisentryweekbeginningweekdayshort = (Gm_Constants::DAYS_SHORT)[0]; 
# $thisentryweekbeginningmonthwordshort = substr($thisentryweekbeginningmonthword, 0, 3);
	my $thisentryweekbeginningmonthwordshort = (Gm_Constants::MONTHS_SHORT)[$thisentryweekbeginningmonth-1];
	
	## TODO: MAKE A GENERIC FUNCTION THAT RETURNS A HAS OF LC AND UC 
	my $thisentryweekbeginningweekdayupper = uc($thisentryweekbeginningweekday);
	my $thisentryweekbeginningmonthwordupper = uc($thisentryweekbeginningmonthword);
	my $thisentryweekbeginningweekdaylower = lc($thisentryweekbeginningweekday);
	my $thisentryweekbeginningmonthwordlower = lc($thisentryweekbeginningmonthword);
	my $thisentryweekbeginningweekdayuppershort = uc($thisentryweekbeginningweekdayshort);
	my $thisentryweekbeginningmonthworduppershort = uc($thisentryweekbeginningmonthwordshort);
	my $thisentryweekbeginningweekdaylowershort = lc($thisentryweekbeginningweekdayshort);
	my $thisentryweekbeginningmonthwordlowershort = lc($thisentryweekbeginningmonthwordshort);

	## TODO: ALLOW USER TO DETERMINE WHICH DAY WEEK START ON, IF DO, NEED TO UPDATE HERE!
# $thisentryweekendingweekday = 'Saturday';
	my $thisentryweekendingweekday = (Gm_Constants::DAYS)[6];
# $thisentryweekendingmonthword = $months[$thisentryweekendingmonth];
	my $thisentryweekendingmonthword = (Gm_Constants::MONTHS)[$thisentryweekendingmonth-1];
# $thisentryweekendingweekdayshort = substr($thisentryweekendingweekday, 0, 3);
	my $thisentryweekendingweekdayshort = (Gm_Constants::DAYS_SHORT)[6];
# $thisentryweekendingmonthwordshort = substr($thisentryweekendingmonthword, 0, 3);
	my $thisentryweekendingmonthwordshort = (Gm_Constants::MONTHS_SHORT)[$thisentryweekendingmonth-1];
	
	## TODO: MAKE A GENERIC FUNCTION THAT RETURNS A HAS OF LC AND UC 
	my $thisentryweekendingweekdayupper = uc($thisentryweekendingweekday);
	my $thisentryweekendingmonthwordupper = uc($thisentryweekendingmonthword);
	my $thisentryweekendingweekdaylower = lc($thisentryweekendingweekday);
	my $thisentryweekendingmonthwordlower = lc($thisentryweekendingmonthword);
	my $thisentryweekendingweekdayuppershort = uc($thisentryweekendingweekdayshort);
	my $thisentryweekendingmonthworduppershort = uc($thisentryweekendingmonthwordshort);
	my $thisentryweekendingweekdaylowershort = lc($thisentryweekendingweekdayshort);
	my $thisentryweekendingmonthwordlowershort = lc($thisentryweekendingmonthwordshort);

	my $gmauthors = Gm_Storage::getAuthors( errHandler=>$errorHandler );
	my $selectedauthor;

	my $thisentryauthoremail = Gm_Constants::EMPTY;
	my $thisentryauthorhomepage = Gm_Constants::EMPTY;
	my $thisentryauthorentrycount = Gm_Constants::EMPTY;

	if( exists( $gmauthors->{$thisentryauthor} )){
		$selectedauthor = $gmauthors->{$thisentryauthor};
		$thisentryauthoremail = $selectedauthor->{'email'};
		$thisentryauthorhomepage = $selectedauthor->{'homepage'};
		$thisentryauthorentrycount = $selectedauthor->{'posttotal'};
	}


# $thisentryfilename = "$EntriesWebPath\/$thisentrynumberpadded\.$entrysuffix";
	my $thisentryfilename = $CONFIGS->{'gmentrieswebpath'}.'/'.
		$thisentrynumberpadded.'.'.$CONFIGS->{'gmentrysuffix'};

	my $thisentrycommentspostlink = "$thisentryfilename\#comments";

#$thisentrycommentstatussmart = $gmsmartlinkmanycommentstemplate;
	## Init to many commentss
	my $thisentrycommentstatussmart = $TEMPLATES->{'gmsmartlinkmanycommentstemplate'};

# if ($thisentrycommentsnumber eq "0") { $thisentrycommentstatussmart = $gmsmartlinknocommentstemplate; }
# if ($thisentrycommentsnumber eq "1") { $thisentrycommentstatussmart = $gmsmartlinkonecommenttemplate; }
	if( $thisentrycommentsnumber < 1 ){ 
		$thisentrycommentstatussmart = $TEMPLATES->{'gmsmartlinknocommentstemplate'};
	} elsif( $thisentrycommentsnumber == 1 ){ 
		$thisentrycommentstatussmart = $TEMPLATES->{'gmsmartlinkonecommenttemplate'}; 
	}

#######
## THESE ARE USED IN GM_FORMATENTRY !!!
	my $thisentrycommentstatussmartupper = uc($thisentrycommentstatussmart);
	my $thisentrycommentstatussmartlower = lc($thisentrycommentstatussmart);
	my $thisentrypagelink = $thisentryfilename;
######

# $indexfilenamesmartcheck = "/$indexfilename";
	my $indexfilenamesmartcheck = '/'.$CONFIGS->{'gmindexfilename'};

# 	my $indexfilenameprefix = substr($indexfilename, 0, 6);
# if ($indexfilenameprefix eq "index.") { $indexfilenamesmartcheck = "/"; }
	if( $CONFIGS->{'gmindexfilename'} =~ m/^\/index\./ ){
		## shortening url automatically
		$indexfilenamesmartcheck = '/';
	}

#######
## THESE ARE USED IN GM_FORMATENTRY !!!	
# $thisentrypageindexlink = "$LogWebPath$indexfilenamesmartcheck";
# $thisentrypagearchiveindexlink = "$EntriesWebPath$indexfilenamesmartcheck";
######
	my $thisentrypageindexlink = $CONFIGS->{'gmlogwebpath'}."$indexfilenamesmartcheck";
	my $thisentrypagearchiveindexlink = $CONFIGS->{'gmentrieswebpath'}."$indexfilenamesmartcheck";

	## TODO: THIS IS FOCUSING HEAVILY ON THE ASSUMPTION THAT THE FILES ARE GOING TO BE
	# PREGENERATED, WHICH IS FINE, BUT IF THIS IS REFACTORED TO BE DYNAMIC, NOTE THIS	AREA!
	my $thisentrypagearchivelogindexlink = '';
	if( $CONFIGS->{'gmkeepmonthlyarchives'} eq Gm_Constants::NO ){
		$thisentrypagearchivelogindexlink = $thisentrypageindexlink;
	} else {
		## NOTE this is where the ending/begining day/month/year vars are used, to create unique archive
		# filenames
		if ($CONFIGS->{'gmarchiveformat'} eq Gm_Constants::WEEK) {
#			$thisentrypagearchivelogindexlink = "$EntriesWebPath/archivo-$thisentryweekbeginningmonthmonth$thisentryweekbeginningdayday$thisentryweekbeginningyearyear-$thisentryweekendingmonthmonth$thisentryweekendingdayday$thisentryweekendingyearyear\.$logarchivesuffix";
			$thisentrypagearchivelogindexlink = $CONFIGS->{'gmentrieswebpath'}.
				"/archivo-$thisentryweekbeginningmonthmonth$thisentryweekbeginningdayday".
				"$thisentryweekbeginningyearyear-$thisentryweekendingmonthmonth".
				"$thisentryweekendingdayday$thisentryweekendingyearyear\.".
				$CONFIGS->{'gmlogarchivesuffix'};

		} else {
#			$thisentrypagearchivelogindexlink = "$EntriesWebPath/archivo-$thisentrymonthmonth$thisentryyearyear\.$logarchivesuffix";
			$thisentrypagearchivelogindexlink = $CONFIGS->{'gmentrieswebpath'}.
				"/archivo-$thisentrymonthmonth$thisentryyearyear\.".$CONFIGS->{'gmlogarchivesuffix'};
		}
	}

#######
## THESE ARE USED IN GM_FORMATENTRY !!!	
# $thisentrypagesmartindexlink = $thisentrypageindexlink;
######	
	my $thisentrypagesmartindexlink = $thisentrypageindexlink;

#if (($thisentrynumber <= $newarchivenumber) && ($keepmonthlyarchives ne Gm_Constants::NO)) {
	if( ($thisentrynumber <= $gmCounters->{'archivetotal'} ) && 
		($CONFIGS->{'gmkeepmonthlyarchives'} ne Gm_Constants::NO) ){
		$thisentrypagesmartindexlink = $thisentrypagearchivelogindexlink;
	}

#######
## THESE ARE USED IN GM_FORMATENTRY !!!	
	my $thisentrycommentslink = Gm_Constants::EMPTY;
	my $thisentrycommentsform = Gm_Constants::EMPTY;
	if( $thisentryallowcomments eq Gm_Constants::YES ){
		if( ($CONFIGS->{'gmallowkarmaorcomments'} eq Gm_Constants::COMMENTS) || 
			($CONFIGS->{'gmallowkarmaorcomments'} eq Gm_Constants::BOTH) ){
# 			$thisentrycommentslink = $gmcommentslinktemplate;
# 			$thisentrycommentsform = $gmcommentsformtemplate;
			$thisentrycommentslink = $TEMPLATES->{'gmcommentslinktemplate'};
			$thisentrycommentsform = $TEMPLATES->{'gmcommentsformtemplate'};
		}
	}

# 	if (($emoticonsallowed eq Gm_Constants::BOTH) || ($emoticonsallowed eq Gm_Constants::COMMENTS)) {
# 			$thisentrycommentsform =~ s/{{smartemoticonscode}}/$gmsmartemoticonscodetemplate/;
# 	} else {
# 			$thisentrycommentsform =~ s/{{smartemoticonscode}}//;
# 	}   
	if( ($CONFIGS->{'gmemoticonsallowed'} eq Gm_Constants::BOTH) || 
		($CONFIGS->{'gmemoticonsallowed'} eq Gm_Constants::COMMENTS) ){
			my $gmsmartemoticonscodetemplate = $TEMPLATES->{'gmsmartemoticonscodetemplate'};
			$thisentrycommentsform =~ s/{{smartemoticonscode}}/$gmsmartemoticonscodetemplate/;
	} else {
			$thisentrycommentsform =~ s/{{smartemoticonscode}}//;
	}  

	#######
	## THESE ARE USED IN GM_FORMATENTRY !!!	
	my $thisentrykarmalink = Gm_Constants::EMPTY;
	my $thisentrykarmaform = Gm_Constants::EMPTY;
	if( $thisentryallowkarma eq Gm_Constants::YES ){
		if( ($CONFIGS->{'gmallowkarmaorcomments'} eq Gm_Constants::KARMA) || 
			($CONFIGS->{'gmallowkarmaorcomments'} eq Gm_Constants::BOTH) ){
			$thisentrykarmalink = $TEMPLATES->{'gmkarmalinktemplate'};
			$thisentrykarmaform = $TEMPLATES->{'gmkarmaformtemplate'};
		}
	}

	if( ($thisentryisanarchive eq Gm_Constants::YES) && 
		($CONFIGS->{'gmposttoarchives'} eq Gm_Constants::NO) ){
		$thisentrykarmalink = Gm_Constants::EMPTY;
		$thisentrykarmaform = Gm_Constants::EMPTY;
		$thisentrycommentslink = Gm_Constants::EMPTY;
		$thisentrycommentsform = Gm_Constants::EMPTY;
	}

	my $thisentrysearchform = $TEMPLATES->{'gmsearchformtemplate'};

#	if ($generateentrypages eq Gm_Constants::NO) {
	if( $CONFIGS->{'gmgenerateentrypages'} eq Gm_Constants::NO ){
		$thisentrycommentslink = Gm_Constants::EMPTY;
		$thisentrycommentsform = Gm_Constants::EMPTY;
		$thisentrysearchform = Gm_Constants::EMPTY;
	}

#######
## THESE ARE USED IN GM_FORMATENTRY !!!
# $thisentrypositivekarmalink = "$cgiwebpath/gm-karma.cgi?vote=positive&amp;entry=$thisentrynumberpadded";
# $thisentrynegativekarmalink = "$cgiwebpath/gm-karma.cgi?vote=negative&amp;entry=$thisentrynumberpadded";
	## TODO: can we remove amp; and just have &
	my $thisentrypositivekarmalink = $CONFIGS->{'gmcgiwebpath'}.
		"/gm-karma.cgi?vote=positive&amp;entry=$thisentrynumberpadded";
	my $thisentrynegativekarmalink = $CONFIGS->{'gmcgiwebpath'}.
		"/gm-karma.cgi?vote=negative&amp;entry=$thisentrynumberpadded";

#######
## THESE ARE USED IN GM_FORMATENTRY !!!
	my $thisentrytotalkarma = $thisentrypositivekarma - $thisentrynegativekarma;
#######

#@thisentrykarmavoters = split (/\|/, $entrylines[1]); ## NOT being used ???

#######
## THESE ARE USED IN GM_FORMATENTRY !!!
	my $thisentrymainbody = $gmEntry->{'maintext'};
	my $thisentrymorebody = $gmEntry->{'extendedtext'};
######

	## TODO: refactor this out so it is done generically, can be reused righ there at least
	if (($thisentrymainbody =~ /\|\*\|\|\*\|/) || ($thisentrymorebody =~ /\|\*\|\|\*\|/)) {
		$thisentrymainbody =~ s/\|\*\|\|\*\|/<PARABREAK>/g;
		$thisentrymorebody =~ s/\|\*\|\|\*\|/<PARABREAK>/g;
	}
	if (($thisentrymainbody =~ /\n\n/) || ($thisentrymorebody =~ /\n\n/)) {
		$thisentrymainbody =~ s/\n\n/<PARABREAK>/g;
		$thisentrymorebody =~ s/\n\n/<PARABREAK>/g;
	}
	my $gmlinebreaktemplate = $TEMPLATES->{'gmlinebreaktemplate'};
	my $gmparaseparationtemplate = $TEMPLATES->{'gmparaseparationtemplate'};
	if (($thisentrymainbody =~ /\|\*\|/) || ($thisentrymorebody =~ /\|\*\|/)) {
		$thisentrymainbody =~ s/\|\*\|/$gmlinebreaktemplate/g;
		$thisentrymorebody =~ s/\|\*\|/$gmlinebreaktemplate/g;
	}
	if (($thisentrymainbody =~ /\n/) || ($thisentrymorebody =~ /\n/)) {
		$thisentrymainbody =~ s/\n/$gmlinebreaktemplate/g;
		$thisentrymorebody =~ s/\n/$gmlinebreaktemplate/g;
	}
	if (($thisentrymainbody =~ /<PARABREAK>/) || ($thisentrymorebody =~ /<PARABREAK>/)) {
		$thisentrymainbody =~ s/<PARABREAK>/$gmparaseparationtemplate/g;
		$thisentrymorebody =~ s/<PARABREAK>/$gmparaseparationtemplate/g;
	}

	## CENSORING  
	## TODO: CAN THIS BE GENERIZIED INTO A UTIL OR WEB FUNCTION
	if( ($CONFIGS->{'gmcensorenabled'} eq Gm_Constants::BOTH) || 
		($CONFIGS->{'gmcensorenabled'} eq Gm_Constants::ENTRIES) ){
	# 	unless ($censorlist eq Gm_Constants::EMPTY) {
	# 		@censoredterms = split(/\n/, $censorlist);
		unless ($CONFIGS->{'gmcensorlist'} eq Gm_Constants::EMPTY) {
			my @censoredterms = split(/\n/, $CONFIGS->{'gmcensorlist'});		
			foreach my $thisterm (@censoredterms) {
				unless ($thisterm eq Gm_Constants::EMPTY) {
					if ((substr($thisterm, 0, 1) eq '[') && (substr($thisterm, -1, 1) eq ']')) {
						my $thisrealterm = $thisterm;
						$thisrealterm =~ s/\[//g;
						$thisrealterm =~ s/\]//g;
						my $thisrealtermlength = length($thisrealterm);
						my $thisrealtermreplace = "*" x $thisrealtermlength;
						my $thisrealtermreplacedash = "-" x $thisrealtermlength;
						if ($thisentrysubject =~ m/$thisrealterm/i) {
							$thisentrysubject =~ s/\b$thisrealterm\b/$thisrealtermreplacedash/isg;
						}
						if ($thisentrymainbody =~ m/$thisrealterm/i) {
							$thisentrymainbody =~ s/\b$thisrealterm\b/$thisrealtermreplace/isg;
						}
						if ($thisentrymorebody =~ m/$thisrealterm/i) {
							$thisentrymorebody =~ s/\b$thisrealterm\b/$thisrealtermreplace/isg;
						}
						if ($thisentryauthor =~ m/$thisrealterm/i) {
							$thisentryauthor =~ s/\b$thisrealterm\b/$thisrealtermreplacedash/isg;
						}
					} else {
						my $thisrealtermlength = length($thisterm);
						my $thisrealtermreplace = "*" x $thisrealtermlength;
						my $thisrealtermreplacedash = "-" x $thisrealtermlength;
						if ($thisentrysubject =~ m/$thisterm/i) {
							$thisentrysubject =~ s/\b$thisterm\b/$thisrealtermreplacedash/isg;
						}
						if ($thisentrymainbody =~ m/$thisterm/i) {
							$thisentrymainbody =~ s/\b$thisterm\b/$thisrealtermreplace/isg;
						}
						if ($thisentrymorebody =~ m/$thisterm/i) {
							$thisentrymorebody =~ s/\b$thisterm\b/$thisrealtermreplace/isg;
						}
						if ($thisentryauthor =~ m/$thisterm/i) {
							$thisentryauthor =~ s/\b$thisterm\b/$thisrealtermreplacedash/isg;
						}
					}
				}
			}
		}
	}

	my $thisentryauthorsmartlink = $thisentryauthor;
## FIX FOR CENSORING?  JUST MOVE AFTER CENSORING PROBABLY
	if( $thisentryauthoremail ne Gm_Constants::EMPTY ){ 
		$thisentryauthorsmartlink = "<a href=\"mailto:$thisentryauthoremail\">$thisentryauthor</a>"; 
	}
	if( $thisentryauthorhomepage ne Gm_Constants::EMPTY ){ 
		my $gmlinktargettemplate = $TEMPLATES->{'gmlinktargettemplate'};
		$thisentryauthorsmartlink = "<a href=\"$thisentryauthorhomepage\" $gmlinktargettemplate>$thisentryauthor</a>"; 
	}

###  PER THE GM MANUAL, THESE ARE DEPRECATED, CLAIMS TO KEEP THEM, BUT THE TEMPLATE VARIABLES ARE GONE (NOT IN 1.3.1)!
# 	my $thisentrymorepreface = Gm_Constants::EMPTY;
# 	my $thisentrymorelink = Gm_Constants::EMPTY;
# 	if( ($thisentrymorebody ne Gm_Constants::EMPTY) && 
# 		($configs->{'gmgenerateentrypages'} ne Gm_Constants::NO) ){
# 		$thisentrymorepreface = $gmmoreprefacetemplate;
# 		$thisentrymorelink = $gmmorelinktemplate;
# 	}

#######
## THESE ARE USED IN GM_FORMATENTRY !!!
	my $thisentrycommentdivider = Gm_Constants::EMPTY;
#	if( $thisentrycommentsnumber ne "0" ){
	if( $thisentrycommentsnumber > 0 ){	
#		$thisentrycommentdivider = $gmcommentdividertemplate;
		$thisentrycommentdivider = $TEMPLATES->{'gmcommentdividertemplate'}
	}

	# changed deprecated <b>, <i>, and <u> tags to corresponding css properties
	# removed code for entrysubject : 1.3.1
	## TODO make generic since we are just doing simple substitutions, Web?  Util?
#	if( ($inlineformatting eq Gm_Constants::ENTRIES) || ($inlineformatting eq Gm_Constants::BOTH) ){
	if( ($CONFIGS->{'gminlineformatting'} eq Gm_Constants::ENTRIES) || 
		($CONFIGS->{'gminlineformatting'} eq Gm_Constants::BOTH) ){
		if( ($thisentrymainbody =~ /\*\*(.*?)\*\*/) || ($thisentrymainbody =~ /\\\\(.*?)\\\\/) || 
			($thisentrymainbody =~ /__(.*?)__/) ){
			$thisentrymainbody =~ s/\*\*(.*?)\*\*/<span style="font-weight: bold;">$1<\/span>/isg;
			$thisentrymainbody =~ s/\\\\(.*?)\\\\/<span style="font-style: italic;">$1<\/span>/isg;
			$thisentrymainbody =~ s/__(.*?)__/<span style="text-decoration: underline;">$1<\/span>/isg;
		}
		if( ($thisentrymorebody =~ /\*\*(.*?)\*\*/) || ($thisentrymorebody =~ /\\\\(.*?)\\\\/) || 
			($thisentrymorebody =~ /__(.*?)__/) ){
			$thisentrymorebody =~ s/\*\*(.*?)\*\*/<span style="font-weight: bold;">$1<\/span>/isg;
			$thisentrymorebody =~ s/\\\\(.*?)\\\\/<span style="font-style: italic;">$1<\/span>/isg;
			$thisentrymorebody =~ s/__(.*?)__/<span style="text-decoration: underline;">$1<\/span>/isg;
		}
	}


#$returnto = Gm_Constants::ENTRIES;
	## checking if emoticons are allowed+--
	if( (($CONFIGS->{'gmemoticonsallowed'} eq Gm_Constants::BOTH) 
		|| ($CONFIGS->{'gmemoticonsallowed'} eq Gm_Constants::COMMENTS)) 
		&& ($gmEntry->{'entryinfo'}{'emoticons'} eq Gm_Constants::YES) ) {
# 		$thisentrymainbody = &gm_emoticons($thisentrymainbody);
# 		$thisentrymorebody = &gm_emoticons($thisentrymorebody);
		$thisentrymainbody = translateEmoticons($thisentrymainbody);
		$thisentrymorebody = translateEmoticons($thisentrymorebody);
	}

	my $thisentrycomments = Gm_Constants::EMPTY;
#	if (($thisentrycommentsnumber ne "0") && ($generateentrypages ne Gm_Constants::NO)) {
	if( ($thisentrycommentsnumber > 0) && 
		($CONFIGS->{'gmgenerateentrypages'} ne Gm_Constants::NO) ){
		#$thisisaspellcheck = Gm_Constants::NO; ## THERE IS NO SPELLCHECKING?  LEFTOFVER?
#		&gm_collatecomments;  ## TODO REFACTOR THIS OUT SINCE ITS DEPRECATED
		$thisentrycomments = collateComments( entry=>$gmEntry, errHandler=>$errorHandler );
		$thisentrycomments =~ s/\&quot;/"/g; ## TODO: MAKE THIS GENERIC UNTIL? WEB?
	}

## TODO: RETURN SOMETHING!!! entrybody, entrymorebody, hash of varius vars?
## TODO: COMMENT OUT COLLATE COMMENTS ONCE EVERYTHING USING THIS ...
## TODO: UPDATE WHAT THIS METHOD DOES !!!

	## returning for format entry to work off of
	$entryVars{'thisentrynumber'} = $thisentrynumber;
	$entryVars{'thisentryauthor'} = $thisentryauthor;
	$entryVars{'thisentrysubject'} = $thisentrysubject;	
	$entryVars{'thisentryweekdaynumber'} = $thisentryweekdaynumber;	
	$entryVars{'thisentrymonth'} = $thisentrymonth;	
	$entryVars{'thisentryday'} = $thisentryday;	
	$entryVars{'thisentryyearyear'} = $thisentryyearyear;	
	$entryVars{'thisentryhour'} = $thisentryhour;	
	$entryVars{'thisentryminute'} = $thisentryminute;	
	$entryVars{'thisentrysecond'} = $thisentrysecond;	
	$entryVars{'thisentryampm'} = $thisentryampm;	
	$entryVars{'thisentrypositivekarma'} = $thisentrypositivekarma;	
	$entryVars{'thisentrynegativekarma'} = $thisentrynegativekarma;	
	$entryVars{'thisentrycommentsnumber'} = $thisentrycommentsnumber;	
	$entryVars{'thisentryallowkarma'} = $thisentryallowkarma;	
	$entryVars{'thisentryallowcomments'} = $thisentryallowcomments;	
	$entryVars{'thisentryopenstatus'} = $thisentryopenstatus;	
	$entryVars{'thisentrymusic'} = $thisentrymusic;	
	$entryVars{'thisentrymood'} = $thisentrymood;	
	$entryVars{'thisentryemoticonsallowed'} = $thisentryemoticonsallowed;	

	$entryVars{'thisentryyear'} = $thisentryyear;
	$entryVars{'thisentryweekday'} = $thisentryweekday;
	$entryVars{'thisentryweekdayshort'} = $thisentryweekdayshort;
	$entryVars{'thisentrymonthword'} = $thisentrymonthword;
	$entryVars{'thisentrymonthwordshort'} = $thisentrymonthwordshort;
	$entryVars{'thisentryweekdayupper'} = $thisentryweekdayupper;
	$entryVars{'thisentrymonthwordupper'} = $thisentrymonthwordupper;
	$entryVars{'thisentryweekdaylower'} = $thisentryweekdaylower;
	$entryVars{'thisentrymonthwordlower'} = $thisentrymonthwordlower;
	$entryVars{'thisentryweekdayuppershort'} = $thisentryweekdayuppershort;
	$entryVars{'thisentrymonthworduppershort'} = $thisentrymonthworduppershort;
	$entryVars{'thisentryweekdaylowershort'} = $thisentryweekdaylowershort;
	$entryVars{'thisentrymonthwordlowershort'} = $thisentrymonthwordlowershort;

	$entryVars{'thisentrymonthmonth'} = $thisentrymonthmonth;
	$entryVars{'thisentrydayday'} = $thisentrydayday;
	$entryVars{'thisentryhourhour'} = $thisentryhourhour;
	$entryVars{'thisentryminuteminute'} = $thisentryminuteminute;
	$entryVars{'thisentrysecondsecond'} = $thisentrysecondsecond;
	$entryVars{'thisentryampmdot'} = $thisentryampmdot;
	$entryVars{'thisentrymilitaryhour'} = $thisentrymilitaryhour;
	$entryVars{'thisentryampmlower'} = $thisentryampmlower;
	$entryVars{'thisentryampmdotlower'} = $thisentryampmdotlower;

	$entryVars{'thisentryweekbeginningmonth'} = $thisentryweekbeginningmonth;
	$entryVars{'thisentryweekbeginningyearyear'} = $thisentryweekbeginningyearyear;
	$entryVars{'thisentryweekendingmonth'} = $thisentryweekendingmonth;
	$entryVars{'thisentryweekendingyearyear'} = $thisentryweekendingyearyear;
	$entryVars{'thisentryweekbeginningday'} = $thisentryweekbeginningday;
	$entryVars{'thisentryweekendingday'} = $thisentryweekendingday;

	$entryVars{'thisentryweekbeginningyear'} = $thisentryweekbeginningyear;
	$entryVars{'thisentryweekendingyear'} = $thisentryweekendingyear;

	$entryVars{'thisentryweekbeginningdayday'} = $thisentryweekbeginningdayday;
	$entryVars{'thisentryweekendingdayday'} = $thisentryweekendingdayday;
	$entryVars{'thisentryweekbeginningmonthmonth'} = $thisentryweekbeginningmonthmonth;
	$entryVars{'thisentryweekendingmonthmonth'} = $thisentryweekendingmonthmonth;
	
	$entryVars{'thisentryweekbeginningweekday'} = $thisentryweekbeginningweekday;
	$entryVars{'thisentryweekbeginningmonthword'} = $thisentryweekbeginningmonthword;
	$entryVars{'thisentryweekbeginningweekdayshort'} = $thisentryweekbeginningweekdayshort;
	$entryVars{'thisentryweekbeginningmonthwordshort'} = $thisentryweekbeginningmonthwordshort;
	$entryVars{'thisentryweekbeginningweekdayupper'} = $thisentryweekbeginningweekdayupper;
	$entryVars{'thisentryweekbeginningmonthwordupper'} = $thisentryweekbeginningmonthwordupper;
	$entryVars{'thisentryweekbeginningweekdaylower'} = $thisentryweekbeginningweekdaylower;
	$entryVars{'thisentryweekbeginningmonthwordlower'} = $thisentryweekbeginningmonthwordlower;
	$entryVars{'thisentryweekbeginningweekdayuppershort'} = $thisentryweekbeginningweekdayuppershort;
	$entryVars{'thisentryweekbeginningmonthworduppershort'} = $thisentryweekbeginningmonthworduppershort;
	$entryVars{'thisentryweekbeginningweekdaylowershort'} = $thisentryweekbeginningweekdaylowershort;
	$entryVars{'thisentryweekbeginningmonthwordlowershort'} = $thisentryweekbeginningmonthwordlowershort;
	
	$entryVars{'thisentryweekendingweekday'} = $thisentryweekendingweekday;
	$entryVars{'thisentryweekendingmonthword'} = $thisentryweekendingmonthword;
	$entryVars{'thisentryweekendingweekdayshort'} = $thisentryweekendingweekdayshort;
	$entryVars{'thisentryweekendingmonthwordshort'} = $thisentryweekendingmonthwordshort;
	
	$entryVars{'thisentryweekendingweekdayupper'} = $thisentryweekendingweekdayupper;
	$entryVars{'thisentryweekendingmonthwordupper'} = $thisentryweekendingmonthwordupper;
	$entryVars{'thisentryweekendingweekdaylower'} = $thisentryweekendingweekdaylower;
	$entryVars{'thisentryweekendingmonthwordlower'} = $thisentryweekendingmonthwordlower;
	$entryVars{'thisentryweekendingweekdayuppershort'} = $thisentryweekendingweekdayuppershort;
	$entryVars{'thisentryweekendingmonthworduppershort'} = $thisentryweekendingmonthworduppershort;
	$entryVars{'thisentryweekendingweekdaylowershort'} = $thisentryweekendingweekdaylowershort;
	$entryVars{'thisentryweekendingmonthwordlowershort'} = $thisentryweekendingmonthwordlowershort;

	$entryVars{'thisentryauthoremail'} = $thisentryauthoremail;
	$entryVars{'thisentryauthorhomepage'} = $thisentryauthorhomepage;
	$entryVars{'thisentryauthorentrycount'} = $thisentryauthorentrycount;
	
	$entryVars{'thisentrycommentspostlink'} = $thisentrycommentspostlink;
	$entryVars{'thisentrycommentstatussmart'} = $thisentrycommentstatussmart;
	$entryVars{'thisentrycommentstatussmartupper'} = $thisentrycommentstatussmartupper;
	$entryVars{'thisentrycommentstatussmartlower'} = $thisentrycommentstatussmartlower;
	$entryVars{'thisentrypagelink'} = $thisentrypagelink;

	$entryVars{'thisentrypageindexlink'} = $thisentrypageindexlink;
	$entryVars{'thisentrypagearchiveindexlink'} = $thisentrypagearchiveindexlink;

	$entryVars{'thisentrypagearchivelogindexlink'} = $thisentrypagearchivelogindexlink;
	$entryVars{'thisentrypagesmartindexlink'} = $thisentrypagesmartindexlink;

	$entryVars{'thisentrycommentslink'} = $thisentrycommentslink;
	$entryVars{'thisentrycommentsform'} = $thisentrycommentsform;	
	$entryVars{'thisentrykarmalink'} = $thisentrykarmalink;
	$entryVars{'thisentrykarmaform'} = $thisentrykarmaform;
	
	$entryVars{'thisentrysearchform'} = $thisentrysearchform;
	$entryVars{'thisentrypositivekarmalink'} = $thisentrypositivekarmalink;
	$entryVars{'thisentrynegativekarmalink'} = $thisentrynegativekarmalink;
	$entryVars{'thisentrytotalkarma'} = $thisentrytotalkarma;
	
	$entryVars{'thisentrymainbody'} = $thisentrymainbody;
	$entryVars{'thisentrymorebody'} = $thisentrymorebody;	
	
	$entryVars{'thisentryauthorsmartlink'} = $thisentryauthorsmartlink;

	$entryVars{'thisentrycommentdivider'} = $thisentrycommentdivider;

	$entryVars{'thisentrycomments'} = $thisentrycomments;

	return( \%entryVars );
}

# ---------------
# Generate Archive Log List
# ---------------
## Generate Archive Log List will create a list of the archive logs
# ARG entryVars => the output of getEntryVariables
# (opt) ARG errHandler => the error handler used to display errors
# RETURN: a string of the archive log list
# DEPRECATES: gm_generatearchiveloglist
# sub generateArchiveLogList {
# 
# 	my $entryVars = $params{'entryVars'};
# 	my $errorHandler = $params{'errHandler'} || \&Gm_Utils::gmWarn;
# # &gm_readcounter;
# 	my $gmCounters = Gm_Storage::getCounters( errHandler=>$errorHandler );
# 
# 	my $thisentrymonth = $entryVars{'thisentrymonth'};
# 	my $thisentryday = $entryVars{'thisentryday'};
# 	my $thisentryhour = $entryVars{'thisentryhour'};
# 	my $thisentryminute = $entryVars{'thisentryminute'};
# 	my $thisentrysecond = $entryVars{'thisentrysecond'};
# 	my $thisentryweekdaynumber = $entryVars{'thisentryweekdaynumber'};
# 	my $thisentryweekday = $entryVars{'thisentryweekday'};
# 	my $thisentrymonthword = $entryVars{'thisentrymonthword'};
# 	my $thisentrymonthmonth = $entryVars{'thisentrymonthmonth'};
# 	my $thisentrydayday = $entryVars{'thisentrydayday'};
# 	my $thisentryhourhour = $entryVars{'thisentryhourhour'};
# 	my $thisentryminuteminute = $entryVars{'thisentryminuteminute'};
# 	my $thisentrysecondsecond = $entryVars{'thisentrysecondsecond'};
# 	my $thisentrymonthmonth = $entryVars{'thisentrymonthmonth'};
# 	my $thisentryhourhour = $entryVars{'thisentryhourhour'};
# 	my $thisentrysecondsecond = $entryVars{'thisentrysecondsecond'};
# 
# 
# ###########################################################
# # 9/3/2001 - CHERYL LAMBERT
# # Problem: If the last archived entry is a different month/year from the current month/year, the
# # {{calendar}} on the Archive Master Index page displays the month/year of the last archived
# # entry instead of the current month/year.  For example, my last archived entry was 8/21/2001
# # but the current date was 9/3/2001 when I discovered this problem.  The {{calendar}} displayed
# # on the Archive Master Index page was for August 2001 not for September 2001.
# # The following section of code corrects this problem by getting the current date and using
# # the current date instead of the last archived date to create the {{calendar}} on the
# # Archive Master Index page.
# 	my ($tsec,$tmin,$thour,$tmday,$tmon,$tyear,$twday,$tyday,$tisdst) = localtime(time);
# 	$tmon += 1;
# 	$tyear += 1900;
# 	if( ($tmon != $thisentrymonth) || ($tyear != $thisentryyearyear) ){  
# 		$thisentrymonth = $tmon;
# 		$thisentryday = $tmday;  
# 		$thisentryhour = $thour;  
# 		$thisentryminute = $tmin;  
# 		$thisentrysecond = $tsec;  
# 		$thisentryweekdaynumber = $twday;  
# # 		$thisentryweekday = $weekdays[$twday];  
# # 		$thisentrymonthword = $months[$thisentrymonth];  
# # 		$thisentrymonthmonth = sprintf ("%2d", $thisentrymonth);  
# # 		$thisentrydayday = sprintf ("%2d", $thisentryday);  
# # 		$thisentryhourhour = sprintf ("%2d", $thisentryhour);  
# # 		$thisentryminuteminute = sprintf ("%2d", $thisentryminute);  
# # 		$thisentrysecondsecond = sprintf ("%2d", $thisentrysecond);  
# # 		$thisentrymonthmonth =~ tr/ /0/;  $thisentrydayday =~ tr/ /0/;  
# # 		$thisentryhourhour =~ tr/ /0/;  $thisentryminuteminute =~ tr/ /0/;  
# # 		$thisentrysecondsecond =~ tr/ /0/;		
# 		$thisentryweekday = (Gm_Constants::DAYS)[$twday];  
# 		$thisentrymonthword = (Gm_Constants::MONTHS)[$tmon];  
# 		$thisentrymonthmonth = Gm_Utils::toTwoDigit( $thisentrymonth );  
# 		$thisentrydayday = Gm_Utils::toTwoDigit( $thisentryday );  
# 		$thisentryhourhour = Gm_Utils::toTwoDigit( $thisentryhour );  
# 		$thisentryminuteminute = Gm_Utils::toTwoDigit( $thisentryminute );  
# 		$thisentrysecondsecond = Gm_Utils::toTwoDigit( $thisentrysecond );  
# 	}
# ###########################################################
# ## Should this permanently affect calling routine? seems like  yes
# 
# 	my $gmentrylist = Gm_Storage::getEntrylist( errHandler=>\&Gm_Web::displayAdminErrorExit );
# 
# 	my $listcountmonthyear = 'begin';
# 	my $listcountweek = 'begin';
# 
# 	my $logarchivelistfinal = Gm_Constants::EMPTY;
# 
# 	foreach my $entry ( sort { $gmentrylist->{$b}{'id'} <=> $gmentrylist->{$a}{'id'} } keys( %$gmentrylist ) ) {
# 		$loglistnumber = $gmentrylist->{$entry}{'id'};
# 		$loglistauthor = $gmentrylist->{$entry}{'author'};
# 		$loglistsubject = $gmentrylist->{$entry}{'subject'};
# 		$loglistdate = $gmentrylist->{$entry}{'created'};
# 		$loglisttimeampm = $gmentrylist->{$entry}{'createt'};
# 		$loglistopenstatus = $gmentrylist->{$entry}{'status'};
# 		$loglistmorestatus = $gmentrylist->{$entry}{'extended'};
# 		&gm_getloglistvariables;
# 	
# 		unless ($loglistopenstatus eq 'C') {
# 	
# 			$thisloglistmonthyear = "$loglistmonthmonth $loglistyearyear";
# 			$thisloglistweek = "$loglistweekbeginningdayday$loglistweekendingdayday";
# 	
# 			if ($archiveformat eq Gm_Constants::WEEK) {
# 				if ($thisloglistweek ne $listcountweek) {
# 					$listsubsub = $gmlogarchiveslinkweeklytemplate;
# 					&entrylistsubsub;
# 					$listsubsub .= "|*|";
# 					$logarchivelistfinal .= $listsubsub;
# 					$listcountweek = $thisloglistweek;
# 				}
# 			} else {
# 				if ($thisloglistmonthyear ne $listcountmonthyear) {
# 					$listsubsub = $gmlogarchiveslinktemplate;
# 					&entrylistsubsub;
# 					$listsubsub .= "|*|";
# 					$logarchivelistfinal .= $listsubsub;
# 					$listcountmonthyear = $thisloglistmonthyear;
# 				}
# 			}
# 	
# 		}
# 	
# 	}
# 
# @loglistfinalcollection = split (/\|\*\|/, $logarchivelistfinal);
# 
# if ($entrylistsortorder eq Gm_Constants::DESCENDING) { @loglistfinalcollection = reverse @loglistfinalcollection; }
# 
# $logarchivelistfinal = join ("$gmlogarchiveslinkseparatortemplate", @loglistfinalcollection);
# 
# }

# ---------------
# Collate Comments
# ---------------
## Collates the Comments into one string, doing template tag replacement, censoring, etc.
# ARG entry => the entry with which to work on (should follow the 
#    entry object format, per Gm_Storage::getEntry
# (opt) ARG errHandler => the error handler used to display errors
# RETURN: string that has all this entries comments 'htmlized'
# DEPRECATES: gm_collatecomments
sub collateComments {
	my (%params) = @_;
	
	## Setting up vars
	my $theEntry = $params{'entry'};
	my $errorHandler = $params{'errHandler'} || \&Gm_Utils::gmWarn;
	my $collatedText = '';  ## this will be returned later
	
	my @entryComments = @{$theEntry->{'comments'}};
	my $commentDisplayNumber = 1;
	
	## Reversing sort order
	if( $CONFIGS->{'gmcommentsorder'} eq Gm_Constants::ASCENDING ){
		@entryComments = reverse @entryComments;
		## TODO MAKE THIS WORK OFF OF COMMENT_ID, since we don't have that if ascending we start from bottom
		$commentDisplayNumber = scalar( @entryComments );
	}

	## Looping through each comment
	foreach my $currComment ( @entryComments ){

		## TODO: GO THROUGH AND REPLACE VARIABLE INSTANCE WITH HASH REFERENCE
		my $thiscommentauthor = $currComment->{'name'};
		my $thiscommentauthorip = $currComment->{'ip'};
		my $thiscommentauthoremailabsolute = $currComment->{'email'};
		my $thiscommentauthorhomepageabsolute = $currComment->{'homepage'};
		my $thiscommentweekdaynumber = $currComment->{'weekday'};
		my $thiscommentmonth = $currComment->{'month'};
		my $thiscommentday = $currComment->{'day'};
		my $thiscommentyearyear = $currComment->{'year'};
		my $thiscommenthour = $currComment->{'hour'};
		my $thiscommentminute = $currComment->{'minute'};
		my $thiscommentsecond = $currComment->{'second'};
		my $thiscommentampm = $currComment->{'ampm'};
		my $thiscommenttext = $currComment->{'comment'};

		my $thiscommentyear = substr($thiscommentyearyear, -2, 2);

		my $thiscommentmonthword = (Gm_Constants::MONTHS)[$thiscommentmonth-1];
		my $thiscommentmonthwordshort = (Gm_Constants::MONTHS_SHORT)[$thiscommentmonth-1];
		my $thiscommentweekday = (Gm_Constants::DAYS)[$thiscommentweekdaynumber];
		my $thiscommentweekdayshort = (Gm_Constants::DAYS_SHORT)[$thiscommentweekdaynumber];

		## This is for tag replacement, TODO: MAKE GENERIC SO RETURNS HAS WITH LC UC VALS?
		my $thiscommentweekdayupper = uc($thiscommentweekday);
		my $thiscommentmonthwordupper = uc($thiscommentmonthword);
		my $thiscommentweekdaylower = lc($thiscommentweekday);
		my $thiscommentmonthwordlower = lc($thiscommentmonthword);
		my $thiscommentweekdayuppershort = uc($thiscommentweekdayshort);
		my $thiscommentmonthworduppershort = uc($thiscommentmonthwordshort);
		my $thiscommentweekdaylowershort = lc($thiscommentweekdayshort);
		my $thiscommentmonthwordlowershort = lc($thiscommentmonthwordshort);

		## This is for tag replacement, TODO: MAKE GENERIC SO RETURNS HAS WITH LC UC VALS?
		my $thiscommentmonthmonth = Gm_Utils::toTwoDigit( $thiscommentmonth );
		my $thiscommentdayday = Gm_Utils::toTwoDigit( $thiscommentday );
		my $thiscommenthourhour = Gm_Utils::toTwoDigit( $thiscommenthour );
		my $thiscommentminuteminute = Gm_Utils::toTwoDigit( $thiscommentminute );
		my $thiscommentsecondsecond = Gm_Utils::toTwoDigit( $thiscommentsecond );

		## Defaults to AM
		my $thiscommentampmdot = 'A.M.';
		my $thiscommentmilitaryhour = $thiscommenthour;  ## TODO: FIX SO 12 AM is 00
		if ($thiscommentampm eq 'PM') {
			$thiscommentampmdot = 'P.M.';
			if ($thiscommenthour ne '12') { 
				$thiscommentmilitaryhour = $thiscommenthour + 12; 
			}
		}
		if (($thiscommenthour eq '12') && ($thiscommentampm eq 'AM')) {
			$thiscommentmilitaryhour = '00';
		}
		$thiscommentmilitaryhour = Gm_Utils::toTwoDigit( $thiscommentmilitaryhour );

		my $thiscommentampmlower = lc($thiscommentampm);
		my $thiscommentampmdotlower = lc($thiscommentampmdot);

		# new  ordinal numbers for {{dayappend}} variable
		my $thiscommentdayappend = Gm_Utils::toOrdinal($thiscommentday); 

		### should we just be making thise webSafe?  TODO, TRY WITH WEB SAFE
		$thiscommentauthor =~ s/"/\&quot;/g;
		$thiscommentauthoremailabsolute =~ s/"/\&quot;/g;
		$thiscommentauthorhomepageabsolute =~ s/"/\&quot;/g;
		
		# TODO: SHOULD LINKBOLDITAL AND LINKONLY BE CONSTANTS?
		# TODO: TAG REPLACEMENT SHOULD BE REMOVED TO CORE HELPER FUNCTION OR UTIL FUNCTION
		if (($CONFIGS->{'gmallowhtmlincomments'} eq 'linkboldital') || 
				($CONFIGS->{'gmallowhtmlincomments'} eq 'linkonly')) {
			if ($thiscommenttext =~ m/\*/i) { 
				$thiscommenttext =~ s/\*/\|AMP\|/g; 
			}
			if (($thiscommenttext =~ m/<a href/i) && ($thiscommenttext =~ m/<\/a>/i)) {
				$thiscommenttext =~ s/<a href/\*a href\*/isg;
				$thiscommenttext =~ s/<\/a>/\*a\*/isg;
			}
			if ($CONFIGS->{'gmallowhtmlincomments'} eq 'linkboldital') {
				if (($thiscommenttext =~ m/<b>/i) && ($thiscommenttext =~ m/<\/b>/i)) {
					$thiscommenttext =~ s/<b>/\*b\*/isg;
					$thiscommenttext =~ s/<\/b>/\*bb\*/isg;
				}
				if (($thiscommenttext =~ m/<i>/i) && ($thiscommenttext =~ m/<\/i>/i)) {
					$thiscommenttext =~ s/<i>/\*i\*/isg;
					$thiscommenttext =~ s/<\/i>/\*ii\*/isg;
				}
			}
		}

		unless ($CONFIGS->{'gmallowhtmlincomments'} eq Gm_Constants::YES) { 
			$thiscommenttext =~ s(<[^>]*>)()gs; 
		}

		## Auto linking magic, own function?
		if ($CONFIGS->{'gmautolinkurls'} eq Gm_Constants::YES) {
			$thiscommenttext =~ s#(^|\s)(\w+://)([A-Za-z0-9?=_\-/.%+&'~\#@!\^]+)#$1<a href="$2$3" rel="nofollow">$2$3</a>#isg;
			$thiscommenttext =~ s#(^|\s)(www.[A-Za-z0-9?=_\-/.%+&'~\#@!\^]+)#$1<a href="http://$2" rel="nofollow" target="_blank">$2</a>#isg;
			$thiscommenttext =~ s/(\w+\@\w+\.\w+)/<a href="mailto:$1">$1<\/a>/isg;
		}

		
		## If emoticons are allowed in both or comments and of course, the local per
		# comment setting overrides so it must be Yes
		if( (($CONFIGS->{'gmemoticonsallowed'} eq Gm_Constants::BOTH) 
 			|| ($CONFIGS->{'gmemoticonsallowed'} eq Gm_Constants::COMMENTS)) 
 		&& ($theEntry->{'entryinfo'}{'emoticons'} eq Gm_Constants::YES) ) {
			$thiscommenttext = translateEmoticons($thiscommenttext);
		}

		## changing text into links and bold or italic, if allowed
		if( ($CONFIGS->{'gmallowhtmlincomments'} eq 'linkboldital') || 
			($CONFIGS->{'gmallowhtmlincomments'} eq 'linkonly') ) {
			if (($thiscommenttext =~ m/\*a href\*/i) && ($thiscommenttext =~ m/\*a\*/i)) {
				$thiscommenttext =~ s/\*a href\*/<a href/isg;
				$thiscommenttext =~ s/\*a\*/<\/a>/isg;
			}
			
			# changed deprecated <b>, and <i> to corresponding css properties
			if ($CONFIGS->{'gmallowhtmlincomments'} eq 'linkboldital') {
				if (($thiscommenttext =~ m/\*b\*/i) && ($thiscommenttext =~ m/\*bb\*/i)) {
					$thiscommenttext =~ s/\*b\*/<span style="font-weight:bold;">/isg;
					$thiscommenttext =~ s/\*bb\*/<\/span>/isg;
				}
				if (($thiscommenttext =~ m/\*i\*/i) && ($thiscommenttext =~ m/\*ii\*/i)) {
					$thiscommenttext =~ s/\*i\*/<span style="font-style:italic;">/isg;
					$thiscommenttext =~ s/\*ii\*/<\/span>/isg;
				}
			}
			
			## We are reversing out ampersand to * translation, TODO: WHY? some sort of sanitizing?
			if ($thiscommenttext =~ m/\|AMP\|/i) { 
				$thiscommenttext =~ s/\|AMP\|/\*/g; 
			}
		}

		## this is doing bold, italic, and underline stranslations TODO: MAKE THIS A FUNCTION IN CORE
		if( ($CONFIGS->{'gminlineformatting'} eq Gm_Constants::COMMENTS) || 
			($CONFIGS->{'gminlineformatting'} eq Gm_Constants::BOTH) ) {
			if( ($thiscommenttext =~ /\*\*(.*?)\*\*/) || ($thiscommenttext =~ /\\\\(.*?)\\\\/) || 
				($thiscommenttext =~ /__(.*?)__/) ) {
				$thiscommenttext =~ s/\*\*(.*?)\*\*/<span style="font-weight:bold;">$1<\/span>/isg;
				$thiscommenttext =~ s/\\\\(.*?)\\\\/<span style="font-style: italic;">$1<\/span>/isg;
				$thiscommenttext =~ s/__(.*?)__/<span style="text-decoration:underline;">$1<\/span>/isg;
			}
		}

		## Are new lines allowed in the comment?
		if ($CONFIGS->{'gmstriplinesfromcomments'} eq Gm_Constants::YES) {
			if (($thiscommenttext =~ /\|\*\|/) || ($thiscommenttext =~ /\n/)) {
				## Making internal newlines and 'real' newlines as spaces.
				$thiscommenttext =~ s/\|\*\|/ /g; 
				$thiscommenttext =~ s/\n/ /g;
			}
		} else {
			if (($thiscommenttext =~ /\|\*\|/) || ($thiscommenttext =~ /\n/)) {
				my $gmlinebreaktemplate = $TEMPLATES->{'gmlinebreaktemplate'};
				my $gmparaseparationtemplate = $TEMPLATES->{'gmparaseparationtemplate'};
				$thiscommenttext =~ s/\|\*\|\|\*\|/<PARABREAK>/g;
				$thiscommenttext =~ s/\n\n/<PARABREAK>/g;
				$thiscommenttext =~ s/\|\*\|/$gmlinebreaktemplate/g;
				$thiscommenttext =~ s/\n/$gmlinebreaktemplate/g;
				$thiscommenttext =~ s/<PARABREAK>/$gmparaseparationtemplate/g;
				$thiscommenttext =~ s/a href/a rel="nofollow" href/g;
				$thiscommenttext =~ s/&(?!#?[xX]?(?:[0-9a-fA-F]+|\w{1,8});)/&amp;/g;
			}
		}

		## TODO: CAN THIS BE GENERIZIED INTO A UTIL OR WEB FUNCTION
		if (($CONFIGS->{'gmcensorenabled'} eq Gm_Constants::BOTH) || 
			($CONFIGS->{'gmcensorenabled'} eq Gm_Constants::COMMENTS)) {
			unless ($CONFIGS->{'gmcensorlist'} eq Gm_Constants::EMPTY) {
				my @censoredterms = split(/\n/, $CONFIGS->{'gmcensorlist'});
				
				## Looping through
				foreach my $thisterm (@censoredterms) {
					unless ($thisterm eq Gm_Constants::EMPTY) {
						if ((substr($thisterm, 0, 1) eq '[') && (substr($thisterm, -1, 1) eq ']')) {
							my $thisrealterm = $thisterm;
							$thisrealterm =~ s/\[//g;
							$thisrealterm =~ s/\]//g;
							my $thisrealtermlength = length($thisrealterm);
							my $thisrealtermreplace = "*" x $thisrealtermlength;
							my $thisrealtermreplacedash = "-" x $thisrealtermlength;
							## Censoring comment
							if ($thiscommenttext =~ m/$thisrealterm/i) {
								$thiscommenttext =~ s/\b$thisrealterm\b/$thisrealtermreplace/isg;
							}
							if ($thiscommentauthor =~ m/$thisrealterm/i) {
								$thiscommentauthor =~ s/\b$thisrealterm\b/$thisrealtermreplacedash/isg;
							}
						} else {
							my $thisrealtermlength = length($thisterm);
							my $thisrealtermreplace = "*" x $thisrealtermlength;
							my $thisrealtermreplacedash = "-" x $thisrealtermlength;
							if ($thiscommenttext =~ m/$thisterm/i) {
								$thiscommenttext =~ s/\b$thisterm\b/$thisrealtermreplace/isg;
							}
							if ($thiscommentauthor =~ m/$thisterm/i) {
								$thiscommentauthor =~ s/\b$thisterm\b/$thisrealtermreplacedash/isg;
							}
						}
					}
				}
			}
		}
		
		### NOTE: this must be done after censoring or smartlink won't pick up censoring
		my $thiscommentauthoremail = Gm_Constants::EMPTY;
		my $thiscommentauthorhomepage = Gm_Constants::EMPTY;

		my $thiscommentauthorsmartlink = $thiscommentauthor;
		if ($thiscommentauthoremailabsolute ne Gm_Constants::EMPTY) {
			$thiscommentauthoremail = $TEMPLATES->{'gmcommentauthoremailtemplate'};
			$thiscommentauthorsmartlink = "<a href=\"mailto:$thiscommentauthoremailabsolute\">$thiscommentauthor</a>";
		}
		if ($thiscommentauthorhomepageabsolute ne Gm_Constants::EMPTY) {
			$thiscommentauthorhomepage = $TEMPLATES->{'gmcommentauthorhomepagetemplate'};
			my $gmcommentlinktargettemplate = $TEMPLATES->{'gmcommentlinktargettemplate'};
			$thiscommentauthorsmartlink = "<a href=\"$thiscommentauthorhomepageabsolute\" $gmcommentlinktargettemplate>$thiscommentauthor</a>";
		}
		

		my $thiscommentfullbody = $TEMPLATES->{'gmcommentstemplate'};
		
		## Time to do some template replacing TODO: RESEARCH IF THIS CAN BE GENERIZIED AND USEFULL TO OTHERS?
		# MAYBE NOT SINCE THESE VALUES WERE ALL JUST SETUP BASED ON THE COMMENT, BUT MAYBE ONLY A FEW PARAMS
		# PASSED TO A GENERIC FUNCTION COULD BE USED??
		if ($thiscommentfullbody =~ m/{{comment/i) {
			$thiscommentfullbody =~ s/{{commentbody}}/$thiscommenttext/gi;
			$thiscommentfullbody =~ s/{{commentauthoremail}}/$thiscommentauthoremail/gi;
			$thiscommentfullbody =~ s/{{commentauthorhomepage}}/$thiscommentauthorhomepage/gi;
			$thiscommentfullbody =~ s/{{commentauthor}}/$thiscommentauthor/gi;
			$thiscommentfullbody =~ s/{{commentauthoremailabsolute}}/$thiscommentauthoremailabsolute/gi;
			$thiscommentfullbody =~ s/{{commentauthorhomepageabsolute}}/$thiscommentauthorhomepageabsolute/gi;
			$thiscommentfullbody =~ s/{{commentauthorsmartlink}}/$thiscommentauthorsmartlink/gi;
			$thiscommentfullbody =~ s/{{commentauthorip}}/$thiscommentauthorip/gi;

			## TODO make this come from comment_id
			$thiscommentfullbody =~ s/{{commentordernumber}}/$commentDisplayNumber/gi; 
		}
		
		## Time template translation
		$thiscommentfullbody =~ s/{{day}}/$thiscommentday/gi;
		$thiscommentfullbody =~ s/{{month}}/$thiscommentmonth/gi;
		$thiscommentfullbody =~ s/{{year}}/$thiscommentyear/gi;
		$thiscommentfullbody =~ s/{{hour}}/$thiscommenthour/gi;
		$thiscommentfullbody =~ s/{{minute}}/$thiscommentminute/gi;
		$thiscommentfullbody =~ s/{{second}}/$thiscommentsecond/gi;
		$thiscommentfullbody =~ s/{{dayday}}/$thiscommentdayday/gi;
		$thiscommentfullbody =~ s/{{monthmonth}}/$thiscommentmonthmonth/gi;
		$thiscommentfullbody =~ s/{{yearyear}}/$thiscommentyearyear/gi;
		$thiscommentfullbody =~ s/{{hourhour}}/$thiscommenthourhour/gi;
		$thiscommentfullbody =~ s/{{minuteminute}}/$thiscommentminuteminute/gi;
		$thiscommentfullbody =~ s/{{secondsecond}}/$thiscommentsecondsecond/gi;
		$thiscommentfullbody =~ s/{{weekday}}/$thiscommentweekday/gi;
		$thiscommentfullbody =~ s/{{weekdayshort}}/$thiscommentweekdayshort/gi;
		$thiscommentfullbody =~ s/{{monthword}}/$thiscommentmonthword/gi;
		$thiscommentfullbody =~ s/{{monthwordshort}}/$thiscommentmonthwordshort/gi;
		$thiscommentfullbody =~ s/{{weekdayupper}}/$thiscommentweekdayupper/gi;
		$thiscommentfullbody =~ s/{{monthwordupper}}/$thiscommentmonthwordupper/gi;
		$thiscommentfullbody =~ s/{{weekdaylower}}/$thiscommentweekdaylower/gi;
		$thiscommentfullbody =~ s/{{monthwordlower}}/$thiscommentmonthwordlower/gi;
		$thiscommentfullbody =~ s/{{weekdayuppershort}}/$thiscommentweekdayuppershort/gi;
		$thiscommentfullbody =~ s/{{monthworduppershort}}/$thiscommentmonthworduppershort/gi;
		$thiscommentfullbody =~ s/{{weekdaylowershort}}/$thiscommentweekdaylowershort/gi;
		$thiscommentfullbody =~ s/{{monthwordlowershort}}/$thiscommentmonthwordlowershort/gi;
		$thiscommentfullbody =~ s/{{militaryhour}}/$thiscommentmilitaryhour/gi;
		$thiscommentfullbody =~ s/{{ampm}}/$thiscommentampm/gi;
		$thiscommentfullbody =~ s/{{ampmdot}}/$thiscommentampmdot/gi;
		$thiscommentfullbody =~ s/{{ampmlower}}/$thiscommentampmlower/gi;
		$thiscommentfullbody =~ s/{{ampmdotlower}}/$thiscommentampmdotlower/gi;
		$thiscommentfullbody =~ s/{{dayappend}}/$thiscommentdayappend/gi;

		$collatedText .= $thiscommentfullbody;

		## Setting the variable to tell us if its the first comment or the Nth comment
		if ( $CONFIGS->{'gmcommentsorder'} eq Gm_Constants::ASCENDING) { 
			$commentDisplayNumber--; 
		} else { 
			$commentDisplayNumber++; 
		}
	} ## End of foreach comment loop

	return( $collatedText );
}


# ------------------
# emoticons subroutine
# ------------------
## Translates the emoticon shorthand (e.g. ':)', ':(' ) to image tags 
# ARG1 = the string to replace the emoticons within
# RETURN: the string with the smiley shorthand translated to html image tags
# DEPRECATES: gm_emoticons
sub translateEmoticons {
	my $line = shift();
	
	my $emoticonspath = $CONFIGS->{'gmemoticonspath'};
	
	$line =~ s/\:\)/<img src=\"$emoticonspath\/smile.gif\" alt=\"smile\" \/>/g;
	$line =~ s/\:O/<img src=\"$emoticonspath\/shocked.gif\" alt=\"shocked\" \/>/g;
	$line =~ s/\:\(/<img src=\"$emoticonspath\/sad.gif\" alt=\"sad\" \/>/g;
	$line =~ s/\:D/<img src=\"$emoticonspath\/biggrin.gif\" alt=\"big grin\" \/>/g;
	$line =~ s/\:P/<img src=\"$emoticonspath\/tongue.gif\" alt=\"razz\" \/>/g;
	$line =~ s/\;\)/<img src=\"$emoticonspath\/wink.gif\" alt=\"wink\" \/>/g;
	$line =~ s/\:angry:/<img src=\"$emoticonspath\/angry.gif\" alt=\"angry, grr\" \/>/g;
	$line =~ s/\:blush:/<img src=\"$emoticonspath\/blush.gif\" alt=\"blush\" \/>/g;
	$line =~ s/\:confused:/<img src=\"$emoticonspath\/confused.gif\" alt=\"confused\" \/>/g;
	$line =~ s/\:cool:/<img src=\"$emoticonspath\/cool.gif\" alt=\"cool eh?\" \/>/g;
	$line =~ s/\:crazy:/<img src=\"$emoticonspath\/crazy.gif\" alt=\"crazy\" \/>/g;
	$line =~ s/\:cry:/<img src=\"$emoticonspath\/cry.gif\" alt=\"crying\" \/>/g;
	$line =~ s/\:doze:/<img src=\"$emoticonspath\/doze.gif\" alt=\"sleepy\" \/>/g;
	$line =~ s/\:hehe:/<img src=\"$emoticonspath\/hehe.gif\" alt=\"hehe\" \/>/g;
	$line =~ s/\:laugh:/<img src=\"$emoticonspath\/laugh.gif\" alt=\"laugh out loud\" \/>/g;
	$line =~ s/\:plain:/<img src=\"$emoticonspath\/plain.gif\" alt=\"plain\" \/>/g;
	$line =~ s/\:rolleyes:/<img src=\"$emoticonspath\/rolleyes.gif\" alt=\"rolls eyes\" \/>/g;
	$line =~ s/\:satisfied:/<img src=\"$emoticonspath\/satisfied.gif\" alt=\"satisfied\" \/>/g;
	
	return( $line );
}


1;
